# Author: Carsten Sachse 08-Jun-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from EMAN2 import EMData, Util
from collections import namedtuple
from filter import filt_table
from fundamentals import rot_shift2D, fft
from morphology import threshold_to_zero, ctf_2, binarize, dilation
from scipy import interpolate
from scipy.interpolate.fitpack2 import InterpolatedUnivariateSpline
from spring.csinfrastr.csdatabase import SpringDataBase, base, SegmentTable, refine_base, RefinementCycleSegmentTable, \
    RefinementCycleTable, HelixTable
from spring.csinfrastr.cslogger import Logger
from spring.micprgs.scansplit import Micrograph
from spring.segment2d.segment import Segment
from spring.segment2d.segmentalign2d import SegmentAlign2d
from spring.segment2d.segmentctfapply import SegmentCtfApply
from spring.segment2d.segmentexam import SegmentExam, SegmentExamMask
from spring.segment3d.refine.sr3d_parameters import SegmentRefine3dReadParameters
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from sqlalchemy.sql.expression import desc
from tabulate import tabulate
from utilities import model_blank, generate_ctf, model_circle
import numpy as np
import os
import shutil


class SegmentRefine3dUnbending(SegmentRefine3dReadParameters):

    def get_ref_session_and_last_cycle(self, ref_cycle_id, ref_db_dir=''):
        ref_db = os.path.join(ref_db_dir, 'refinement{0:03}.db'.format(ref_cycle_id))
        temp_ref_db = os.path.join(self.tempdir, 'ref_temp_{0:03}.db'.format(ref_cycle_id))
        shutil.copy(ref_db, temp_ref_db)
        ref_session = SpringDataBase().setup_sqlite_db(refine_base, temp_ref_db)
        
        last_cycle = ref_session.query(RefinementCycleTable).order_by(desc(RefinementCycleTable.id)).first()
            
        return ref_session, temp_ref_db, last_cycle
    

    def get_all_helices_from_last_refinement_cycle(self, ref_cycle_id, spring_db):
        session = SpringDataBase().setup_sqlite_db(base, spring_db)
        current_ref_db = os.path.join(os.path.dirname(spring_db), 'refinement{0:03}.db'.format(ref_cycle_id))
        if os.path.exists(current_ref_db):
            ref_session, temp_ref_db, last_cycle = self.get_ref_session_and_last_cycle(ref_cycle_id, os.path.dirname(spring_db))
        else:
            ref_session = None
            temp_ref_db = None
            last_cycle = None
        
        helices = session.query(HelixTable).order_by(HelixTable.id).all()
        
        return helices, session, ref_session, temp_ref_db, last_cycle
    

    def get_segments_from_helix(self, session, each_helix):
        each_helix_segments = session.query(SegmentTable.stack_id).filter(SegmentTable.helix_id == each_helix.id).all()
    
        return each_helix_segments
    

    def get_segment_ids_from_helix(self, session, each_helix):
        each_helix_segments = self.get_segments_from_helix(session, each_helix)
        each_helix_segments, = list(zip(*each_helix_segments))
        
        return each_helix_segments


    def get_all_segments_from_refinement_cycle(self, ref_session, last_cycle, helix_segments):
        each_ref_segments = ref_session.query(RefinementCycleSegmentTable).\
        filter(RefinementCycleSegmentTable.cycle_id == last_cycle.id).\
        filter(RefinementCycleSegmentTable.stack_id.in_(helix_segments)).\
        order_by(RefinementCycleSegmentTable.id).all()
        
        return each_ref_segments
    

    def get_avg_helix_shift_x_and_y_and_avg_inplane_angles(self, orientation_result):
        avg_helix_x_shifts = np.array([each_result.lavg_helix_shift_x_A for each_result in orientation_result])
        helix_y_shifts = np.array([each_result.helix_shift_y_A for each_result in orientation_result])
        inplane_angles = np.array([each_result.lavg_inplane_angle for each_result in orientation_result])
        
        return avg_helix_x_shifts, helix_y_shifts, inplane_angles
    

    def sort_data_and_discard_duplicates(self, x, y):
        """
         * http://mail.scipy.org/pipermail/scipy-user/2011-February/028341.html
        """
        # Sort data
        j = np.argsort(x)
        x = x[j]
        y = y[j]
    
        # De-duplicate data
        mask = np.r_[True, (np.diff(x) > 0)]
        if not mask.all():
            pass
            # could do something smarter here
        x = x[mask]
        y = y[mask]
        
        return x, y
    

    def extrapolate_second_order_to_end_of_box(self, xcoord, ycoord, avg_rot_angle, alignment_size, stepsize):
        """
        >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
        >>> s = SegmentRefine3d()
        >>> s.extrapolate_second_order_to_end_of_box(np.arange(10) + 10, np.arange(10) + 5, 45, 5 * np.sqrt(2), np.sqrt(2)) #doctest: +NORMALIZE_WHITESPACE
        (array([  6.,   7.,   8.,   9.,  10.,  11.,  12.,  13.,  14.,  15.,  16.,
                17.,  18.,  19.,  20.,  21.,  22.,  23.]), 
                array([  1.,   2.,   3.,   4.,   5.,   6.,   7.,   8.,   9.,  10.,  11.,
                12.,  13.,  14.,  15.,  16.,  17.,  18.]))
        """
        rotx, roty = Segment().rotate_coordinates_by_angle(xcoord, ycoord, avg_rot_angle, xcoord[0], ycoord[0])
        
        rotx, roty = self.sort_data_and_discard_duplicates(rotx, roty)
        
        left_append = (rotx[0] - np.arange(0, alignment_size, stepsize)[1:])[::-1]
        rotx_append = np.insert(rotx, np.zeros(len(left_append)), left_append)
        
        right_append = (rotx[-1] + np.arange(0, alignment_size, stepsize))[1:]
        rotx_extra = np.append(rotx_append, right_append)
        
        spline = InterpolatedUnivariateSpline(rotx, roty, k=1)
        roty_extra = spline(rotx_extra)
        
        roty_extra = np.polyval(np.polyfit(rotx_extra, roty_extra, 2), rotx_extra)
        
        xcoord_extra, ycoord_extra = Segment().rotate_coordinates_by_angle(rotx_extra, roty_extra, -avg_rot_angle, 
        xcoord[0], ycoord[0])
        
        return xcoord_extra, ycoord_extra
        

    def get_picked_segment_coordinates(self, each_helix_segments, pixelsize):
        hel_x_coordinates = np.array([each_segment.picked_x_coordinate_A / pixelsize 
                                      for each_segment in each_helix_segments])
        hel_y_coordinates = np.array([each_segment.picked_y_coordinate_A / pixelsize 
                                      for each_segment in each_helix_segments])
        
        return hel_x_coordinates, hel_y_coordinates
    

    def fill_in_excluded_quantities_by_interpolation(self, each_helix_segments, each_segment, unbending_angles,
    x_coord, y_coord, each_sel_helix_segments, avg_rot_angle, complete_x, complete_y):
        
        rotx, roty = Segment().rotate_coordinates_by_angle(x_coord, y_coord, avg_rot_angle, complete_x[0], complete_y[0])
        
        comp_rotx, comp_roty = Segment().rotate_coordinates_by_angle(complete_x, complete_y, avg_rot_angle,
        complete_x[0], complete_y[0])
        
        try:
            spline = InterpolatedUnivariateSpline(rotx, roty, k=2)
            int_comp_roty = spline(comp_rotx)
        except:
            int_comp_roty = np.polyval(np.polyfit(rotx, roty, 2), comp_rotx)
            
#            rotx, roty = self.sort_data_and_discard_duplicates(rotx, roty)
#            spline = InterpolatedUnivariateSpline(rotx, roty, k=2)
        
        x_coord, y_coord = Segment().rotate_coordinates_by_angle(comp_rotx, int_comp_roty, -avg_rot_angle,
        complete_x[0], complete_y[0])
        
        distances = np.array([each_segment.distance_from_start_A for each_segment in each_helix_segments])
        sel_distances = np.array([each_segment.distance_from_start_A for each_segment in 
                each_sel_helix_segments])
        
        spline = InterpolatedUnivariateSpline(sel_distances, unbending_angles, k=2)
        unbending_angles = spline(distances)

        return x_coord, y_coord, unbending_angles


    def get_cut_coordinates_named_tuple(self):
        cut_coord = namedtuple('cut_coord', 'x_coord y_coord unbending_angle second_order_fit helix_id')

        return cut_coord



    def get_helix_coordinates_named_tuple(self):
        helix_coord = namedtuple('helix_coord', 'x_coord y_coord')
        
        return helix_coord
    

    def get_helices_coordinates_required_for_unbending_from_database(self, prev_ref_cycle_id, each_binfactor,
    info_series, segment_stack, pixelsize, spring_db='spring.db'):
        large_segment = EMData()
        large_segment.read_image(segment_stack)
        large_segment_size = large_segment.get_xsize()
        
        helices, session, ref_session, temp_ref_db, last_cycle = \
        self.get_all_helices_from_last_refinement_cycle(prev_ref_cycle_id, spring_db)
        
        cut_coordinates = []
        helices_coordinates = []
        
        cut_coord = self.get_cut_coordinates_named_tuple()
        helix_coord = self.get_helix_coordinates_named_tuple()
        
        for each_helix_id, each_helix in enumerate(helices):
            each_helix_segments = session.query(SegmentTable).filter(SegmentTable.helix_id == each_helix.id).all()
            each_helix_stack_ids = [each_segment.stack_id for each_segment in each_helix_segments]
            
            second_order_fits = np.array([each_segment.second_order_fit for each_segment in each_helix_segments])
#            if each_binfactor == info_series[0].bin_factor:
#                if second_order_fits[0] is not None:
#                    second_order_fits /= pixelsize
#            else:
            second_order_fits = np.array([None] * len(second_order_fits))
            
            unbending_angles = [-each_segment.lavg_inplane_angle for each_segment in each_helix_segments]
            
            complete_hel_x_coordinates, complete_hel_y_coordinates = \
            self.get_picked_segment_coordinates(each_helix_segments, pixelsize)
            
            inplane_angles = [each_segment.inplane_angle for each_segment in each_helix_segments]
            avg_ip_angle = inplane_angles[int(len(inplane_angles) / 2)] - 90.0
            
            x_coordinates = complete_hel_x_coordinates
            y_coordinates = complete_hel_y_coordinates
            if last_cycle is not None:
                helix_segment_count = len(each_helix_stack_ids)
                
                each_ref_helix_segments = ref_session.query(RefinementCycleSegmentTable).\
                filter(RefinementCycleSegmentTable.cycle_id == last_cycle.id).\
                filter(RefinementCycleSegmentTable.selected == True).\
                filter(RefinementCycleSegmentTable.stack_id.in_(each_helix_stack_ids)).\
                order_by(RefinementCycleSegmentTable.id).all()
                
                each_ref_helix_stack_ids = [each_segment.stack_id for each_segment in each_ref_helix_segments]
                    
                each_sel_helix_segments = session.query(SegmentTable).\
                filter(SegmentTable.stack_id.in_(each_ref_helix_stack_ids)).all()
                    
                select_ratio = len(each_ref_helix_segments) / float(helix_segment_count)
                if select_ratio > 0.4:
                    
                    hel_x_coordinates, hel_y_coordinates = self.get_picked_segment_coordinates(each_sel_helix_segments,
                    pixelsize)
                    
                    avg_helix_x_shifts, helix_y_shifts, inplane_angles = \
                    self.get_avg_helix_shift_x_and_y_and_avg_inplane_angles(each_ref_helix_segments)
                    
                    fitted_x_shifts, fitted_y_shifts = \
                    SegClassReconstruct().compute_sx_sy_from_shifts_normal_and_parallel_to_helix_axis_with_inplane_angle(
                    avg_helix_x_shifts, helix_y_shifts, inplane_angles)
                    
                    x_coordinates = hel_x_coordinates + fitted_x_shifts
                    y_coordinates = hel_y_coordinates + fitted_y_shifts
                    unbending_angles = (-inplane_angles) % 360
                    
                    if len(complete_hel_x_coordinates) > len(x_coordinates):
                        x_coordinates, y_coordinates, unbending_angles =\
                        self.fill_in_excluded_quantities_by_interpolation(each_helix_segments, each_segment,
                        unbending_angles, x_coordinates, y_coordinates, each_sel_helix_segments, avg_ip_angle,
                        complete_hel_x_coordinates, complete_hel_y_coordinates)
        
                    self.log.ilog('Helix {0} will be unbent using refined coordinates.'.format(each_helix.id))
                ref_session.close()
                os.remove(temp_ref_db)
            else:
                self.log.ilog('Helix {0} will be unbent using picked coordinates.'.format(each_helix.id))
            
            extra_x_coord, extra_y_coord = self.extrapolate_second_order_to_end_of_box(x_coordinates, y_coordinates, 
            avg_ip_angle, large_segment_size, self.stepsize / pixelsize)
                    
#            import matplotlib.pyplot as plt
#            plt.plot(complete_hel_x_coordinates, complete_hel_y_coordinates, 'o', x_coordinates, y_coordinates, '.',
#            extra_x_coord, extra_y_coord, 'x')
#            plt.savefig('tt.pdf')
            
            for each_x_coord, each_y_coord, each_angle, each_fit in zip(x_coordinates, y_coordinates, unbending_angles,
            second_order_fits):
                cut_coordinates.append(cut_coord(each_x_coord, each_y_coord, each_angle, each_fit, each_helix.id))
                
            helices_coordinates.append(helix_coord(extra_x_coord, extra_y_coord))
        
        return helices_coordinates, cut_coordinates
    

    def make_unbending_named_tuple(self):
        unbending = namedtuple('info', 'angle shift_x shift_y')
                
        return unbending
                

    def prepare_unique_updated_stack_name(self, segment_stack, ref_cycle_id):
        large_straightened_segment_stack = '{0}{1}{2}{3}'.format(self.tempdir,
        os.path.splitext(os.path.basename(segment_stack))[0], ref_cycle_id, 
        os.path.splitext(os.path.basename(segment_stack))[-1])

        return large_straightened_segment_stack


    def unbend_window_and_mask_input_stack(self, segment_stack, ref_cycle_id, pixelinfo,
    previous_parameters, mask_params, helices_coordinates, cut_coordinates, masked_segment_stack, resolution_aim):
        self.log.fcttolog()
        self.log.in_progress_log()
        large_segment = EMData()
        large_segment.read_image(segment_stack)
        large_segment_size = large_segment.get_xsize()
        
        large_straightened_segment_stack = self.prepare_unique_updated_stack_name(segment_stack, ref_cycle_id)
        masked_segment_stack = os.path.join(self.tempdir, masked_segment_stack)
        
        unique_tilts, ideal_power_imgs, circle_mask, padsize = \
        self.prepare_binary_layer_line_filters_if_layer_line_filtering_demanded(resolution_aim, ref_cycle_id - 1,
        pixelinfo, self.helical_symmetries[0], self.rotational_symmetry_starts[0])
        
        rectangular_mask = SegmentExam().make_smooth_rectangular_mask(pixelinfo.helixwidthpix, pixelinfo.helix_heightpix,
        pixelinfo.alignment_size)
        
        unbending = self.make_unbending_named_tuple()
        unbending_info = []
        unbending_loginfo = []
        for each_index, each_parameter in enumerate(previous_parameters):

            cut_coord = cut_coordinates[each_parameter.local_id] 
            
            current_coordinates = (cut_coord.x_coord, cut_coord.y_coord)
#             large_segment.read_image(segment_stack, each_parameter.local_id)
            large_segment, frame_stack_names = \
            self.read_large_segment_average_if_running_avg_frames_requested(segment_stack, large_segment,
            each_parameter)

            hel_x = helices_coordinates[cut_coord.helix_id - 1].x_coord
            hel_y = helices_coordinates[cut_coord.helix_id - 1].y_coord
            
            helix_shift_x, helix_shift_y = \
            SegClassReconstruct().compute_distances_to_helical_axis(each_parameter.shift_x, each_parameter.shift_y,
            each_parameter.inplane_angle)
             
            shift_x, shift_yy = \
            SegClassReconstruct().compute_sx_sy_from_shifts_normal_and_parallel_to_helix_axis_with_inplane_angle(
            helix_shift_x, 0.0, -each_parameter.inplane_angle)

            shifted_helix_coord = zip(np.array(hel_x) - shift_x, np.array(hel_y) - shift_yy)
            
            large_straightened_segment, straightened_fit, central_ip_angle = \
            Segment().unbend_segment_using_coordinates(large_segment, large_segment_size, shifted_helix_coord,
            current_coordinates, (cut_coord.x_coord, cut_coord.y_coord), (-cut_coord.unbending_angle)%360,
            cut_coord.second_order_fit)
            
            unbending_loginfo += [[each_parameter.stack_id, each_parameter.local_id, straightened_fit[0],
            straightened_fit[1], straightened_fit[2], self.layer_line_filter]]
            
#            previous_parameters[each_index] = previous_parameters[each_index]._replace(psi=270.0, shift_x=0.0,
#            shift_y=helix_shift_y)
            
            previous_parameters[each_index] = previous_parameters[each_index]._replace(psi=270.0, shift_x=0.0,
            shift_y=helix_shift_y)
            
            unbending_info.append(unbending(central_ip_angle, each_parameter.shift_x, each_parameter.shift_y))
            
            if self.layer_line_filter:# and resolution_aim in ['high', 'max']:
                closest_tilt_id = np.argmin(np.abs(unique_tilts - mask_params[each_index].out_of_plane_tilt))
                
                large_straightened_segment = \
                SegClassReconstruct().filter_image_by_fourier_filter_while_padding(large_straightened_segment,
                large_segment_size, padsize, ideal_power_imgs[closest_tilt_id])
                
            straightened_segment = Util.window(large_straightened_segment, pixelinfo.alignment_size,
            pixelinfo.alignment_size, 1, 0, 0, 0)

            straightened_segment *= rectangular_mask
            straightened_segment.write_image(masked_segment_stack, each_parameter.local_id)
            large_straightened_segment.write_image(large_straightened_segment_stack, each_parameter.local_id)
            
        header = ['stack_id', 'local_id', 'a*x^2', 'b*x', 'c', 'layer-line_filter']
        msg = tabulate(unbending_loginfo, header)
        self.log.tlog('The following segments were straightened according to the square ' + \
        'function:\n{0}'.format(msg))
            
        self.remove_frames_if_running_avg_frames_requested(frame_stack_names)

        return previous_parameters, unbending_info, masked_segment_stack, segment_stack, large_straightened_segment_stack
    

class SegmentRefine3dPreparation(SegmentRefine3dUnbending):

    def define_all_segmentrefine3d_parameters(self, p):
        self.define_input_output_iteration(p)
        self.define_refinement_strategy_options(p)
        self.define_helical_symmetry_parameters(p)
        self.define_alignment_parameters(p)
        self.define_selection_parameters(p)
        self.define_filter_parameters(p)
        self.define_reconstruction_parameters(p)
        self.define_mpi_parameters(p)


    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters

            self.define_all_segmentrefine3d_parameters(p)
    

class SegmentRefine3dSymmetry(SegmentRefine3dPreparation):
#     def compute_zsection_percentage_from_width(self, pixelsize, helixwidth, volume_radius):
#         """
#         >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
#         >>> s = SegmentRefine3d()
#         >>> s.compute_zsection_percentage_from_width(1, 50, 50)
#         0.8660254037844386
#         """
#         section_z = np.sqrt(volume_radius ** 2 - ((helixwidth / float(pixelsize)) ** 2 / 4)) / float(volume_radius)
#         
#         return section_z
#     
#     
#     def compute_zsection_percentage_from_rise(self, pixelsize, helical_rise, z_height):
#         """
#         >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
#         >>> s = SegmentRefine3d()
#         >>> s.compute_zsection_percentage_from_rise(1, 40, 100)
#         0.8
#         >>> s.compute_zsection_percentage_from_rise(1, 50, 100)
#         1.0
#         """
#         z_height_pix = z_height * pixelsize
#         multiple_rise = int(z_height_pix / float(helical_rise))
#         section_z = (helical_rise * multiple_rise) / float(z_height_pix)
#         
#         return section_z
#     
# 
#     def symmetrize_volume_old(self, volume, pixelsize, helical_symmetry, point_symmetry, helixwidth, min_zsection=None):
#         helical_rise, helical_rotation = helical_symmetry
#         if helical_rise != 0:
#             volume_radius = volume.get_zsize() / 2
#             z_height = volume.get_zsize() 
#             if min_zsection is None:
#                 section_z = min(self.compute_zsection_percentage_from_width(pixelsize, helixwidth, volume_radius), 
#                         self.compute_zsection_percentage_from_rise(pixelsize, helical_rise, z_height) - 0.01,
#                         0.8)
#             else:
#                 section_z = min_zsection
#                 
#             self.log.ilog('The volume will be helically symmetrized: {0}, {1}, {2}, {3}, {4}'.format(pixelsize, 
#             helical_rise, helical_rotation, section_z, z_height) + ' (pixelsize, rise, rotation, z_section, z_height)')
#             
#             try:
#                 helix_volume_helicise = volume.helicise(pixelsize, helical_rise, -helical_rotation, section_z)
#             except:
#                 helix_volume_helicise = volume.helicise(1, helical_rise / float(pixelsize), -helical_rotation,
#                 section_z)
#         elif helical_rise == 0 and helical_rotation == 0: 
#             helix_volume_helicise = volume.copy()
#         elif helical_rise == 0:
#             multiple = abs(int(360.0 / helical_rotation + 0.5))
#             helix_volume_helicise = volume.symvol('c{0}'.format(multiple))
#             
#         if point_symmetry != 'c1':
#             helix_volume_helicise = helix_volume_helicise.symvol(point_symmetry)
#             
#         return helix_volume_helicise
#     
#  
#     def generate_helical_volume_of_projection_size(self, reference_volume, reconstruction_size, alignment_size,
#     helical_symmetry, pixelsize, point_symmetry, helixwidth, symmetrize=True):
#         reference_volume, padded_in_z = self.adjust_volume_dimension_by_padding_or_windowing(reference_volume,
#         reconstruction_size, alignment_size)
#         
#         assert reference_volume.get_xsize() == reconstruction_size 
#         assert reference_volume.get_ysize() == reconstruction_size 
#         assert reference_volume.get_zsize() == alignment_size
#             
#         if padded_in_z and symmetrize:
#             reference_volume = self.symmetrize_volume(reference_volume, pixelsize, helical_symmetry, point_symmetry, 
#             helixwidth)
#             
#         return reference_volume
    

    def adjust_volume_dimension_by_padding_or_windowing(self, reference_volume, reconstruction_size, alignment_size):
        volsize_x = reference_volume.get_xsize()
        volsize_y = reference_volume.get_ysize()
        volsize_z = reference_volume.get_zsize()
        if volsize_x > reconstruction_size:
            reference_volume = Util.window(reference_volume, reconstruction_size, reference_volume.get_ysize(),
            reference_volume.get_zsize(), 0, 0, 0)
        elif volsize_x < reconstruction_size:
            reference_volume = Util.pad(reference_volume, reconstruction_size, reference_volume.get_ysize(),
            reference_volume.get_zsize(), 0, 0, 0, 'average')
        if volsize_y > reconstruction_size:
            reference_volume = Util.window(reference_volume, reference_volume.get_xsize(), reconstruction_size,
            reference_volume.get_zsize(), 0, 0, 0)
        elif volsize_y < reconstruction_size:
            reference_volume = Util.pad(reference_volume, reference_volume.get_xsize(), reconstruction_size,
            reference_volume.get_zsize(), 0, 0, 0, 'average')
        padded_in_z = False
        if volsize_z > alignment_size:
            reference_volume = Util.window(reference_volume, reference_volume.get_xsize(), reference_volume.get_ysize(),
            alignment_size, 0, 0, 0)
        elif volsize_z < alignment_size:
            reference_volume = Util.pad(reference_volume, reference_volume.get_xsize(), reference_volume.get_ysize(),
            alignment_size, 0, 0, 0, 'average')
            
            padded_in_z = True

        return reference_volume, padded_in_z


    def get_sizes_that_increase_by_percent(self, cur_zsize, max_zsize, percent):
        """
        >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
        >>> s = SegmentRefine3d()
        >>> s.get_sizes_that_increase_by_percent(20, 100, 0.2)
        [24, 28, 34, 40, 48, 58, 70, 84, 100]
        >>> s.get_sizes_that_increase_by_percent(100, 100, 0.2)
        [100]
        """
        inc = int(cur_zsize * percent)
        sizes = list(range(cur_zsize, max_zsize, inc))
        
        per_sizes = []
        per_size = cur_zsize
        even = 0
        for each_size in sizes:
            per_size += int(per_size * percent)
            if per_size %2 != even:
                per_size +=1
            per_sizes.append(per_size)
            
        vol_sizes = [int(each_size) for each_size in per_sizes if each_size < max_zsize] + [max_zsize]
        
        return vol_sizes
        

    def symmetrize_volume(self, vol, pixelsize, section_size, rise, rotation):
        if rise != 0:
            vol = vol.helicise(pixelsize, rise, -rotation, section_size)
        elif rise == 0 and rotation == 0:
            vol = vol.copy()
        elif rise == 0:
            multiple = abs(int(360.0 / float(rotation) + 0.5))
            vol = vol.symvol('c{0}'.format(multiple))
        
        return vol
        

    def symmetrize_long_volume_in_steps(self, vol, helical_symmetry, pixelsize, xy_size, z_size):
        section_size = 0.8
        rise, rotation = helical_symmetry
        z_sizes = self.get_sizes_that_increase_by_percent(vol.get_zsize(), z_size, 1 - section_size)
        sections = [0.7] + (len(z_sizes) - 1) * [section_size]
        for each_zsize, each_section_size in zip(z_sizes, sections):
            vol, padded_in_z = self.adjust_volume_dimension_by_padding_or_windowing(vol, xy_size, each_zsize)
            vol = self.symmetrize_volume(vol, pixelsize, each_section_size, rise, rotation)
            
        return vol


    def generate_long_helix_volume(self, vol, xy_size, z_size, helical_symmetry, pixelsize, point_symmetry):
        vol = self.symmetrize_long_volume_in_steps(vol, helical_symmetry, pixelsize, xy_size, z_size)
        
        hel_vol = vol.copy()
        if point_symmetry != 'c1':
            apix_attr = ['apix_x', 'apix_y', 'apix_z']
            apix_vals = [hel_vol.get_attr(each_attr) for each_attr in apix_attr]
            
            hel_vol = hel_vol.symvol(point_symmetry)
            [hel_vol.set_attr(each_attr, each_val) for each_val, each_attr in zip(apix_vals, apix_attr)]
            
        return hel_vol
    

    def compute_rec_size_for_helix(self, helixwidth, helical_rise, pixelsize):
        """
        >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
        >>> SegmentRefine3d().compute_rec_size_for_helix(200, 1.408, 1.2)
        220
        """
        rec_size_in_Angstrom = 2 * np.sqrt((1.1 * helixwidth / 2.0) ** 2 + (max(helical_rise, 60.0)) ** 2)
        reconstruction_size = rec_size_in_Angstrom / pixelsize
        if reconstruction_size >= 32:
            reconstruction_size = Segment().determine_boxsize_closest_to_fast_values(reconstruction_size)
        else:
            reconstruction_size = int(reconstruction_size)
        
        return reconstruction_size
    
    
    def compute_alignment_size_in_pixels(self, alignment_size_in_A, pixelsize):
        alignment_size = alignment_size_in_A / pixelsize
        if alignment_size >= 32:
            alignment_size = Segment().determine_boxsize_closest_to_fast_values(alignment_size)
        else:
            alignment_size = int(alignment_size)
        
        return alignment_size


    def compute_alignment_and_reconstruction_size_in_pixels(self, alignment_size_in_A, helical_rise, helixwidth_in_A,
    pixelsize):
        """
        # values for fast boxsizes from http://blake.bcm.edu/emanwiki/EMAN2/BoxSize
        >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
        >>> s = SegmentRefine3d()
        >>> s.compute_alignment_and_reconstruction_size_in_pixels(200, 1.408, 100, 1.0)
        (220, 168)
        >>> s.compute_alignment_and_reconstruction_size_in_pixels(200, 1.408, 300, 1.0)
        (352, 352)
        """
        reconstruction_size = self.compute_rec_size_for_helix(helixwidth_in_A, helical_rise, pixelsize)
        alignment_size = self.compute_alignment_size_in_pixels(alignment_size_in_A, pixelsize)
        alignment_size = max(alignment_size, reconstruction_size)
        
        return alignment_size, reconstruction_size 
    

    def rescale_reference_volume_in_case_vol_pixelsize_differs_from_current_pixelsize(self, reconstruction_size,
    reference_volume, pixelsize, alignment_size=None):
        if alignment_size is None:
            alignment_size = reconstruction_size
        if reference_volume.has_attr('apix_z'):
            if reference_volume.get_attr('apix_z') != 1.0:
                ref_pixelsize = reference_volume.get_attr('apix_z')
                scaling_factor = ref_pixelsize / pixelsize
                if scaling_factor > 1:
                    reference_volume, padded_in_z = \
                    self.adjust_volume_dimension_by_padding_or_windowing(reference_volume,
                    reconstruction_size, alignment_size)
                    reference_volume.scale(scaling_factor)
                elif scaling_factor < 1:
                    reference_volume.scale(scaling_factor)

        return reference_volume


    def get_ref_file_name(self, ref_id):
        reference_file = '{prefix}_ref_mod{ref_id:03}{ext}'.format(prefix=os.path.splitext(self.outfile_prefix)[0],
        ref_id=ref_id, ext=os.path.splitext(self.outfile_prefix)[-1])

        return reference_file


    def make_reference_info_named_tuple(self):
        return namedtuple('ref_info', 'model_id ref_file prj_stack fine_prj_stack helical_symmetry point_symmetry ' + \
                          'rotational_symmetry fsc')


    def prepare_reference_volumes(self):
        rises = np.array([each_hel_sym[0] for each_hel_sym in self.helical_symmetries])
        rotations = np.array([each_hel_sym[1] for each_hel_sym in self.helical_symmetries])
        
        reference_volume = EMData()
        reference_info = []
        ref_info_nt = self.make_reference_info_named_tuple()

        if len(set(self.polar_helices)) == 1:
            self.polar_helix = list(set(self.polar_helices))[0]
        else:
            self.polar_helix = 'polar'

        point_symmetries = [] 
        for each_ref_id in list(range(len(self.helical_symmetries))):
            point_symmetries.append(SegClassReconstruct().determine_point_group_symmetry_from_input(self.polar_helices[each_ref_id],
            self.rotational_symmetry_starts[each_ref_id]))
        
        read_sym = True
        if len(self.references) == 1 and len(self.helical_symmetries) > 1:
            self.references = len(self.helical_symmetries) * self.references
            read_sym = False

        for each_ref_id, (each_helical_symmetry, each_point_sym, each_rot_sym) in enumerate(zip(
        self.helical_symmetries, point_symmetries, self.rotational_symmetry_starts)):
            alignment_size, reconstruction_size = \
            self.compute_alignment_and_reconstruction_size_in_pixels(self.alignment_size_in_A, each_helical_symmetry[0],
            self.helixwidth, self.ori_pixelsize)
        
            if self.reference_option:
                reference_volume.read_image(self.references[each_ref_id])
                if reference_volume.has_attr('helical rise') and reference_volume.has_attr('helical rotation') and read_sym:
                    vol_rise = reference_volume.get_attr('helical rise')
                    vol_rot = reference_volume.get_attr('helical rotation')
                    
                    closest_sym_id = np.argmin(np.sqrt((rises - vol_rise) ** 2 + (rotations - vol_rot) ** 2))
                    helical_symmetry = self.helical_symmetries[closest_sym_id]
                    point_sym = point_symmetries[closest_sym_id]
                    rotational_symmetry = self.rotational_symmetry_starts[closest_sym_id]
                else:
                    helical_symmetry = each_helical_symmetry
                    point_sym = each_point_sym
                    rotational_symmetry = each_rot_sym

                reference_volume = \
                self.rescale_reference_volume_in_case_vol_pixelsize_differs_from_current_pixelsize(reconstruction_size,
                reference_volume, self.ori_pixelsize, alignment_size)

            else:
                helix_inner_widthpix = int(round(self.helix_inner_width / self.ori_pixelsize))
                helixwidthpix = int(round(self.helixwidth / self.ori_pixelsize))
                reference_volume = SegClassReconstruct().make_smooth_cylinder_mask(helixwidthpix,
                helix_inner_widthpix, alignment_size, width_falloff=0.1)
                helical_symmetry = each_helical_symmetry
                rotational_symmetry = each_rot_sym
                point_sym = each_point_sym

            reference_file = self.get_ref_file_name(each_ref_id)
            reference_volume.write_image(reference_file)

            reference_info.append(ref_info_nt(each_ref_id, reference_file, None, None, helical_symmetry, point_sym,
            rotational_symmetry, None))
            
        return reference_info


    def prepare_3dctf_avg_squared(self, reconstruction_size, pixelsize):
        
        temp_db = self.copy_spring_db_to_tempdir()
        session = SpringDataBase().setup_sqlite_db(base, temp_db)
        
        matched_segment = session.query(SegmentTable).filter(SegmentTable.stack_id == 0).first()
        session.close()
        os.remove(temp_db)

        if matched_segment.ctf_convolved:
            if matched_segment.ctftilt_applied:
                choice = 'ctftilt'
            if matched_segment.ctffind_applied:
                choice = 'ctffind'
            
            astigmatism_option = True
            
            ctf_parameters = SegmentCtfApply().get_ctf_values_from_database_and_compute_local_ctf_based_if_demanded(choice,
            'convolve', astigmatism_option, pixelsize, self.spring_path)
            
            first_ctf = generate_ctf(ctf_parameters[0])
            ctf1d_length = len(ctf_2(reconstruction_size, first_ctf))
            avg_ctf_squared_profile = np.zeros(ctf1d_length)
            for each_ctf_param in ctf_parameters:
                each_ctf = generate_ctf(each_ctf_param)
                each_ctf_2_profile = np.array(ctf_2(reconstruction_size, each_ctf))
                avg_ctf_squared_profile += each_ctf_2_profile
            
            avg_ctf_squared_profile /= float(len(ctf_parameters))
                
            ctf3d_avg_squared = model_blank(reconstruction_size, reconstruction_size, reconstruction_size)
            ctf3d_avg_squared.set_value_at(0, 0, 0, 1)
            ctf3d_avg_squared = fft(filt_table(ctf3d_avg_squared, avg_ctf_squared_profile.tolist()))
            ctf3d_avg_squared_file = self.tempdir + 'ctf3d_avg_squared.hdf'
            ctf3d_avg_squared.write_image(ctf3d_avg_squared_file)
        elif matched_segment.ctf_phase_flipped:
            ctf3d_avg_squared_file = None
            self.ctf_correction = False
            self.log.wlog('3D CTF correction was demanded but according to specified spring.db segments were ' + \
            'previously phase-flipped. Therefore, no further 3D CTF correction required.')
        elif not matched_segment.ctf_phase_flipped and not matched_segment.ctf_convolved:
            msg = 'You requested 3D CTF correction but according to the provided spring.db your input segments ' + \
            'were not CTF corrected (neither phase-flipped nor convolved). Please perform CTF ' + \
            'correction in the program \'segment\' or do not request 3D CTF correction option.'
            raise ValueError(msg)
            
        return ctf3d_avg_squared_file
    

    def prepare_binary_layer_line_filters_if_layer_line_filtering_demanded(self, resolution_aim, each_iteration_number,
    pixelinfo, helical_symmetry, rotational_sym):
        if self.layer_line_filter:# and resolution_aim in ['high', 'max']:
            angular_blur = self.compute_angular_blur_based_on_Crowther_criterion(each_iteration_number,
            self.alignment_size_in_A, pixelinfo.pixelsize)
            
#            angular_blur = None
            projection_parameters = self.generate_Euler_angles_for_projection(self.azimuthal_angle_count,
            self.out_of_plane_tilt_angle_range, self.out_of_plane_tilt_angle_count, helical_symmetry[1])
            
            unique_tilts, padsize, ideal_power_imgs = \
            self.generate_binary_layer_line_filters_including_angular_blur(projection_parameters, pixelinfo,
            helical_symmetry, rotational_sym, angular_blur / 5.0)
            
            circle_mask = model_circle(padsize / 2, padsize, padsize)
        else:
            unique_tilts = None
            ideal_power_imgs = None
            circle_mask = None
            padsize = None
            
        return unique_tilts, ideal_power_imgs, circle_mask, padsize
    


    def add_tempdir_directory_to_filename(self, tempdir, segment_stack, masked_segment_stack):
        local_segment_stack = os.path.join(tempdir, os.path.basename(segment_stack))
        masked_segment_stack = os.path.join(tempdir, os.path.basename(masked_segment_stack))
        
        return local_segment_stack, masked_segment_stack
    

    def get_frame_stack_path(self, ori_large_segment_stack, each_frame_id):
        frame_stack_name = os.path.splitext(ori_large_segment_stack)[0] + '-{0}'.format(each_frame_id) + \
        os.path.splitext(ori_large_segment_stack)[-1]

        frame_stack_path = os.path.join(self.tempdir, os.path.basename(frame_stack_name))
        return frame_stack_path


    def read_large_segment_average_if_running_avg_frames_requested(self, segment_stack, large_segment, each_parameter):
        frame_stack_names = []
        if self.frame_motion_corr and self.frame_avg_window > 2:
            frame_segment = EMData()
            for each_frame_id in list(range(self.frame_avg_window)):
                stack_name = self.get_frame_stack_path(segment_stack, each_frame_id)
                frame_stack_names.append(stack_name)
                if each_frame_id == 0:
                    large_segment.read_image(stack_name, each_parameter.local_id)
                else:
                    frame_segment.read_image(stack_name, each_parameter.local_id)
                    large_segment += frame_segment
        else:
            large_segment.read_image(segment_stack, each_parameter.local_id)

        return large_segment, frame_stack_names


    def remove_frames_if_running_avg_frames_requested(self, frame_stack_names):
        if self.frame_motion_corr and self.frame_avg_window > 2:
            for each_frame_stack in frame_stack_names:
                os.remove(each_frame_stack)


    def window_and_mask_input_stack(self, segment_stack, pixelinfo, mask_params, masked_segment_stack, each_info,
    ref_cycle_id):
        self.log.fcttolog()
        self.log.in_progress_log()
        large_segment = EMData()
        large_segment.read_image(segment_stack)
        large_segment_size = large_segment.get_xsize()
        
        masked_segment_stack = os.path.join(self.tempdir, masked_segment_stack)
        if self.layer_line_filter:
            local_segment_stack = self.prepare_unique_updated_stack_name(segment_stack, ref_cycle_id)
        else:
            local_segment_stack = None
        
        rectangular_mask = SegmentExam().make_smooth_rectangular_mask(pixelinfo.helixwidthpix, pixelinfo.helix_heightpix,
        large_segment_size)
        
        masking_log_info = []
        
        unique_tilts, ideal_power_imgs, circle_mask, padsize = \
        self.prepare_binary_layer_line_filters_if_layer_line_filtering_demanded(each_info.resolution_aim,
        ref_cycle_id, pixelinfo, self.helical_symmetries[0], self.rotational_symmetry_starts[0])
            
        for each_parameter in mask_params:
            large_segment, frame_stack_names = \
            self.read_large_segment_average_if_running_avg_frames_requested(segment_stack, large_segment,
            each_parameter)

            if self.layer_line_filter:# and each_info.resolution_aim in ['high', 'max']:
                closest_tilt_id = np.argmin(np.abs(unique_tilts - each_parameter.out_of_plane_tilt))
                
                rotated_binary_filter = rot_shift2D(ideal_power_imgs[closest_tilt_id],
                -each_parameter.lavg_inplane_angle)
                
                rotated_binary_filter = binarize(rotated_binary_filter, 0.01)
                rotated_binary_filter *= circle_mask
#                rotated_binary_filter.write_image(self.tempdir + 'layer_filter.hdf', each_parameter.local_id)

                large_segment = SegClassReconstruct().filter_image_by_fourier_filter_while_padding(large_segment,
                large_segment_size, padsize, rotated_binary_filter)
                
                large_segment.write_image(local_segment_stack, each_parameter.local_id)
            
            rotated_rectangular_mask = Segment().shift_and_rotate_image(rectangular_mask,
            -each_parameter.lavg_inplane_angle, each_parameter.lavg_helix_shift_x, 0)
            
            large_segment *= rotated_rectangular_mask
            segment = Util.window(large_segment, pixelinfo.alignment_size, pixelinfo.alignment_size, 1, 0, 0, 0)
            segment.write_image(masked_segment_stack, each_parameter.local_id)
            
            masking_log_info += [[each_parameter.stack_id, each_parameter.local_id,
            (-each_parameter.lavg_inplane_angle)%360, each_parameter.lavg_helix_shift_x, self.layer_line_filter]]
            
        msg = tabulate(masking_log_info, list(each_parameter._fields)[0:4] + ['layer line filter'])
        self.log.tlog('The following segments were masked by a rotated rectangle according to their previous ' + \
                      'picking parameters:\n{0}'.format(msg))
            
        self.remove_frames_if_running_avg_frames_requested(frame_stack_names)
                
        return masked_segment_stack, segment_stack, local_segment_stack
            
            
    def build_structural_mask_from_volume(self, reference_vol, helixwidthpix, helix_inner_width_pix, pixelsize,
    sigma_factor=1.0, width_falloff=0.03):
        
        reconstruction_size = reference_vol.get_xsize()
        alignment_size = reference_vol.get_zsize()
        
        cylinder_mask = SegClassReconstruct().make_smooth_cylinder_mask(helixwidthpix, helix_inner_width_pix,
        reconstruction_size, alignment_size, width_falloff)
        
        stat = Micrograph().get_statistics_from_image(reference_vol, cylinder_mask)
        
        binary_vol = binarize(reference_vol * cylinder_mask, sigma_factor * stat.sigma)
        if width_falloff != 0:
            dilation_pixels = 7
            odd = 1
            if (dilation_pixels) % 2 != odd:
                dilation_pixels += 1
            dilation_mask = model_blank(dilation_pixels, dilation_pixels, dilation_pixels, bckg = 1.0)
            structural_mask = dilation(binary_vol, dilation_mask)
        else:
            structural_mask = binary_vol
        
        return structural_mask
    
    
    def compute_prj_size_from_max_out_of_plane_tilt_and_diameter(self, alignment_size, tilt_range, helixwidthpix):
        """
        >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
        >>> s = SegmentRefine3d()
        >>> s.compute_prj_size_from_max_out_of_plane_tilt_and_diameter(600, [-12, 12], 180)
        675
        """
        max_tilt = max([abs(each_extrema) for each_extrema in tilt_range])
        single_walled_distance_at_end = helixwidthpix * np.sin(np.deg2rad(max_tilt))
        projection_size = alignment_size + int(round(2 * single_walled_distance_at_end))
        
        return projection_size
    
    
    def prepare_volume_for_projection_by_masking_and_thresholding(self, resolution_aim, reference_volume,
    pixelinfo, helical_symmetry, point_symmetry):
        vol_x = reference_volume.get_xsize()
        vol_z = reference_volume.get_zsize()
        if resolution_aim in ['low', 'medium']: 
            vol_mask = SegClassReconstruct().make_smooth_cylinder_mask(pixelinfo.helixwidthpix,
            pixelinfo.helix_inner_widthpix, vol_x, vol_z, width_falloff=0.1)
            
        elif resolution_aim in ['high', 'max']:
            vol_mask = self.build_structural_mask_from_volume(reference_volume, pixelinfo.helixwidthpix,
            pixelinfo.helix_inner_widthpix, pixelinfo.pixelsize)
            
            vol_mask = SegmentExamMask().add_smooth_gaussian_falloff_to_edge_of_binary_mask(vol_x, 0.05, vol_mask)
            
        stat = Micrograph().get_statistics_from_image(reference_volume, vol_mask)
        reference_volume *= vol_mask
        reference_volume = threshold_to_zero(reference_volume, stat.avg)
            
        projection_size = self.compute_prj_size_from_max_out_of_plane_tilt_and_diameter(pixelinfo.alignment_size,
        self.out_of_plane_tilt_angle_range, pixelinfo.helixwidthpix)
        
        reference_volume = self.generate_long_helix_volume(reference_volume, pixelinfo.reconstruction_size,
        projection_size, helical_symmetry, pixelinfo.pixelsize, point_symmetry)

#         reference_volume = self.generate_helical_volume_of_projection_size(reference_volume,
#         pixelinfo.reconstruction_size, projection_size, helical_symmetry, pixelinfo.pixelsize, point_symmetry,
#         self.helixwidth)
        
        return projection_size, reference_volume, stat.sigma
    

    def apply_square_root_fsc_filter_to_coefficients(self, fsc_lines, filter_coefficients, projection_size,
    custom_filter_file, pixelsize):
        if self.fsc_filter and fsc_lines is not None:
            fsc_arr = np.array(fsc_lines.cylinder_masked)
            fsc_arr[fsc_arr < 0] = 0
            fsc_arr = np.sqrt(fsc_arr)
            
            res = SegmentExam().make_oneoverres(filter_coefficients, pixelsize)
            f = interpolate.interp1d(np.linspace(res[0], res[-1], len(fsc_arr)), fsc_arr)
            res_int = np.linspace(res[0], res[-1], len(res))
            fsc_int = f(res_int)
            first_005 = np.where(fsc_int < 0.05)[0]
            
            if len(first_005) != 0:
                if res_int[first_005[0]] > 0.05:
                    fsc_int[first_005[0]:] = 0.0
                else:
                    fsc_int = SegmentAlign2d().prepare_filter_function(self.high_pass_filter_option,
                    self.high_pass_filter_cutoff, self.low_pass_filter_option, 0.05, pixelsize,
                    projection_size, 0.08, self.custom_filter_option, custom_filter_file, self.bfactor)
                    fsc_int = np.array(fsc_int)
            
            filter_coefficients = np.array(filter_coefficients) * fsc_int
            filter_coefficients = filter_coefficients.tolist()
            log_pairs = list(zip(res, filter_coefficients))
            log_info = tabulate(log_pairs, ['Resolution', 'filter_coefficients'])
            
            self.log.ilog('The reference volume is filtered by the square root of the FSC curve:{0}'.format(log_info))
            
        return filter_coefficients
    

    def rescale_freq_filter_columns(self, ori_pixelsize, rescaled_dimension, rescaled_apix, spat_freq, fourier_coeff):
        """
        >>> from spring.segment3d.refine.sr3d_main import SegmentRefine3d
        >>> s = SegmentRefine3d()
        >>> freq = np.linspace(0, 0.5, 100)
        >>> coeff = np.abs(np.cos(np.linspace(0, np.pi, 100)))
        >>> rescaled_freq, rescaled_coeff = s.rescale_freq_filter_columns(1.2, 44, 2.4, freq, coeff)
        >>> len(rescaled_freq)
        22
        >>> rescaled_coeff
        array([ 1.        ,  0.99725998,  0.98905491,  0.97542979,  0.95645925,
                0.93224727,  0.90292654,  0.86865772,  0.82962861,  0.78605309,
                0.73816997,  0.68624164,  0.63055267,  0.57140824,  0.50913246,
                0.44406661,  0.37656726,  0.3070043 ,  0.23575894,  0.1632216 ,
                0.08978981,  0.01586596])
        """
        binfactor = rescaled_apix / ori_pixelsize
        reduced_length = int(len(fourier_coeff) / binfactor)
        red_fourier_coefficients = fourier_coeff[:reduced_length]
        red_spat_freq = np.linspace(spat_freq[0], spat_freq[-1], reduced_length)
        
        spline = InterpolatedUnivariateSpline(red_spat_freq, red_fourier_coefficients)
        rescaled_spat_freq = np.linspace(spat_freq[0], spat_freq[-1], rescaled_dimension / 2)
        rescaled_f_coeff = np.abs(spline(rescaled_spat_freq))
    
        return rescaled_spat_freq, rescaled_f_coeff
    

    def rescale_custom_filter_file(self, custom_filter_file, ori_pixelsize, rescaled_dimension, rescaled_apix):
        
        ffile = open(custom_filter_file, 'r')
        coeff_lines = ffile.readlines()
        spat_freq = np.array([float(each_line.split()[0]) for each_line in coeff_lines])
        fourier_coeff = np.array([float(each_line.split()[-1]) for each_line in coeff_lines])
        ffile.close()
        
        rescaled_spat_freq, rescaled_f_coeff = self.rescale_freq_filter_columns(ori_pixelsize, rescaled_dimension,
        rescaled_apix, spat_freq, fourier_coeff)
        
        dat_str = '\n'.join(['{0}\t{1}'.format(each_freq, each_f_coeff) 
                             for each_freq, each_f_coeff in zip(rescaled_spat_freq, rescaled_f_coeff)])
        
        rescaled_filter_file = '{0}_rescaled.dat'.format(os.path.splitext(custom_filter_file)[0])
        ffile = open(rescaled_filter_file, 'w')
        ffile.write(dat_str)
        ffile.close()
        
        return rescaled_filter_file
        

    def write_out_reference_volume(self, reference_file, each_iteration_number, ref_cycle_id, reference_volume,
    model_id=None):
        if each_iteration_number == 1:
            os.remove(reference_file.ref_file)
        
        if model_id is None:
            ref_file_name = '{prefix}_ref_{iter:03}{ext}'.format(prefix=os.path.splitext(self.outfile_prefix)[0], 
            iter=ref_cycle_id, ext=os.path.splitext(self.outfile_prefix)[-1])
        else:
            ref_file_name = '{prefix}_mod_{mod:03}_ref_{iter:03}{ext}'.format(prefix=os.path.splitext(self.outfile_prefix)[0], 
            mod=model_id, iter=ref_cycle_id, ext=os.path.splitext(self.outfile_prefix)[-1])

        reference_file = reference_file._replace(ref_file = ref_file_name)

        reference_volume.write_image(reference_file.ref_file)

        return reference_file


    def filter_and_mask_reference_volume(self, resolution_aim, each_reference, pixelinfo, fsc_lines=None):
        self.log.fcttolog()
        
        reference_volume = EMData()
        reference_volume.read_image(each_reference.ref_file)

        projection_size, reference_volume, sigma = \
        self.prepare_volume_for_projection_by_masking_and_thresholding(resolution_aim, reference_volume, pixelinfo,
        each_reference.helical_symmetry, each_reference.point_symmetry)
                                      
        if self.high_pass_filter_option or self.low_pass_filter_option or self.custom_filter_option or \
        self.bfactor != 0 or self.fsc_filter:
            if self.custom_filter_option:
                custom_filter_file = self.rescale_custom_filter_file(self.custom_filter_file, self.ori_pixelsize,
                projection_size, pixelinfo.pixelsize)
            else:
                custom_filter_file = self.custom_filter_file
                
            filter_coefficients = SegmentAlign2d().prepare_filter_function(self.high_pass_filter_option,
            self.high_pass_filter_cutoff, self.low_pass_filter_option, self.low_pass_filter_cutoff, pixelinfo.pixelsize,
            projection_size, 0.08, self.custom_filter_option, custom_filter_file, self.bfactor)
            
            if fsc_lines is not None:
                filter_coefficients = self.apply_square_root_fsc_filter_to_coefficients(fsc_lines, filter_coefficients,
                projection_size, custom_filter_file, pixelinfo.pixelsize)
            
            if self.custom_filter_option:
                os.remove(custom_filter_file)
                
            reference_volume = filt_table(reference_volume, filter_coefficients)
                                                                
        return reference_volume
        
        


# Author: Carsten Sachse
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to extract particles from micrographs 
"""
from collections import namedtuple
from spring.csinfrastr.csdatabase import SpringDataBase, base
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segment import SegmentPar, Segment
from tabulate import tabulate
import os

class ParticleStackPar(SegmentPar):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'particlestack'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.segment_features = Features()
        self.feature_set = self.segment_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
    
    def set_center_and_remove_ends_for_helices(self):
        pass
#         self.feature_set = self.set_center_option(self.feature_set)

    def set_helix_or_particle_centering(self):
        self.set_center_and_remove_ends_for_helices()
        
    def set_segment_or_window_features(self):
        self.feature_set = self.set_particle_coordinate_files(self.feature_set)
        self.feature_set = self.set_window_size(self.feature_set)
        self.feature_set = self.segment_features.set_particle_inner_and_outer_diameter(self.feature_set)
        
    def define_program_states(self):
        self.feature_set.properties['Number of lines Logfile']=6000

        self.feature_set.program_states['assign_reorganize']='Initialize micrographs and segments to convert them ' + \
        'into Spring\'s file structure'
        self.feature_set.program_states['readmic']='Loading new micrograph'
        self.feature_set.program_states['center_segments']='Segments are centered in their x and y position'
        self.feature_set.program_states['window_segment']='Windowing particles from micrograph'

    def set_particle_coordinate_files(self, feature_set):
        inp3 = 'Particle coordinates'
        feature_set.parameters[inp3] = 'scan034.box'
        feature_set.properties[inp3] = feature_set.file_properties(1000, ['box', 'txt'], 'getFiles')
        feature_set.hints[inp3] = 'Input: file with identical name of corresponding micrograph (accepted file ' + \
        'formats EMAN\'s Boxer and EMAN2\'s *.%s coordinates)' % (', '.join(self.properties[inp3].ext))
        feature_set.level[inp3]='beginner'

        return feature_set
    
        
    def set_window_size(self, feature_set):
        inp4 = 'Window size in Angstrom'
        feature_set.parameters[inp4] = int(600)
        feature_set.hints[inp4] = 'Size should be 2 x particle diameter'
        feature_set.properties[inp4] = feature_set.Range(100, 1500, 50)
        feature_set.level[inp4]='beginner'
        
        return feature_set
    

    def set_center_option(self, feature_set):
        inp7 = 'Center option'
        feature_set.parameters[inp7] = bool(True)
        feature_set.hints[inp7] = 'Particle are centered with respect to their x and y position'
        feature_set.level[inp7]='intermediate'

        return feature_set


class ParticleStack(Segment):
    def define_segment_or_window_parameters(self, p):
        self.coordinate_files_entry = p['Particle coordinates']
        self.window_size = p['Window size in Angstrom']
        self.inner_diameter, self.outer_diameter = p['Estimated inner and outer particle diameter in Angstrom']
    
    def make_micinfo_named_tuple(self):
        micinfo = namedtuple('micinfo', 'mic_id micrograph coordinate_list picked_coordinates')
        
        return micinfo
    
    
    def single_out(self, pair):
        micrograph_coordinates = []
        micinfo = self.make_micinfo_named_tuple()
        for mic_id, (micfile, coordfile, overlap) in enumerate(pair):
            coord_line = self.get_picked_coordinates_from_file(coordfile, overlap)
            
            xcoord = []
            ycoord = []
            for each_coordinate_line in ''.join(coord_line).splitlines():
                xcenter, ycenter = self.convert_coordinate_line_to_xy_coordinates(each_coordinate_line)
                xcoord.append(xcenter)
                ycoord.append(ycenter)
            
            coord = zip(xcoord, ycoord)
            micrograph_coordinates.append(micinfo(mic_id, micfile, coordfile, coord))
        
        return micrograph_coordinates
        

    def prepare_windowing(self):
        pairs = self.assign_reorganize(self.micrograph_files, self.coordinate_files)
        micrograph_coordinates = self.single_out(pairs)
        
        return micrograph_coordinates
    
    
    def excise_particles_from_micrographs(self, micrograph_coordinates):
        if self.spring_db_option:
            session = SpringDataBase().setup_sqlite_db(base, self.spring_path)
        else:
            session = None

        micrograph_img, matched_mic_find, micrograph_name, log_info, ctf_info, each_image_index = \
        self.pre_loop_setup()
        
        for each_mic_index, each_mic in enumerate(micrograph_coordinates):
            
            micrograph_img, matched_mic_find, micrograph_name = self.read_micrograph_and_get_ctf_info(micrograph_img,
            matched_mic_find, micrograph_name, session, each_mic)
            
            for (xcoord, ycoord) in each_mic.picked_coordinates:
                
                windowed_segment, ctf_info = self.window_segment_and_invert_normalize_and_ctf_correct(session,
                self.segsizepix, each_mic, micrograph_img, matched_mic_find, xcoord, ycoord, ctf_info, each_image_index)
                
                windowed_segment.write_image(self.outfile, each_image_index)
                
                log_info += [[each_image_index, each_mic.mic_id, micrograph_name, xcoord, ycoord, self.normoption,
                self.invertoption]]
                
                each_image_index = each_image_index + 1
            self.log.plog(100 * (each_mic_index + 1) / len(micrograph_coordinates) - 10)
            
        msg = tabulate(log_info, ['Particle_id', 'mic_id', 'micrograph', 'x_coordinate', 'y_coordinate',
        'normalization', 'contrast_inversion'])

        if self.ctfcorrect_option:
            msg += '\n' + tabulate(ctf_info, self.get_ctfinfo_header())
        self.log.ilog(log_info)
                
        return self.outfile
    
                
    def window_particles(self):
        micrograph_coordinates = self.prepare_windowing()
        self.log.plog(10)
        imgstack = self.excise_particles_from_micrographs(micrograph_coordinates)
        if self.binoption is True: 
            binned_imgstack = '{0}-{1}xbin{2}'.format(os.path.splitext(imgstack)[0], self.binfactor,
            os.path.splitext(imgstack)[-1])
            
            self.binstack(imgstack, binned_imgstack, self.binfactor)
        
        self.log.endlog(self.feature_set)
        
    
def main():
    # Option handling
    parset = ParticleStackPar()
    mergeparset = OptHandler(parset)
 
    ######## Program
    mic = ParticleStack(mergeparset)
    mic.window_particles()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Author: Carsten Sachse 3-Jan-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Module to generate rst files for sphinx documentation
"""

from EMAN2 import EMData, EMUtil, EMNumPy, Util
from glob import glob
import os
import shutil
from spring.csinfrastr.csproductivity import Support
import subprocess
import sys

import numpy as np


class GalleryTestSetup(object):
    def prepare_tests_in_new_directory(self):
        os.mkdir(self.program)
        os.chdir(self.program)

    def convert_wildcards_if_present_to_correct_filename(self, filename):
        if '?' in filename or '*' in filename:
            filename = glob(filename)[0]
            
        return filename

    def copy_input_and_output_files_to_images(self):
        self.infile = list(self.parameters.values())[0]
        self.outfile = list(self.parameters.values())[1]
        self.infile = self.convert_wildcards_if_present_to_correct_filename(self.infile)
        self.outfile = self.convert_wildcards_if_present_to_correct_filename(self.outfile)

class GalleryTestsMicrograph(object):
    def perform_test_scansplit(self):
        from spring.tests.micprgs.test_scansplit import TestScanSplitMain
        self.test = TestScanSplitMain()
        self.test.setup()
        self.test.do_test_case_ss1()

    def perform_test_micexam(self):
        from spring.tests.micprgs.test_micexam import TestMicrographExamMain
        self.test = TestMicrographExamMain()
        self.test.setup()
        self.test.do_test_case_se1()
        
    def perform_test_michelixtracegrid(self):
        from spring.tests.micprgs.test_michelixtrace import TestMicHelixTraceMore
        self.test = TestMicHelixTraceMore()
        self.test.setup()
        self.test.do_test_case_mt3()
        key_outfile = list(self.test.feature_set.parameters.keys())[1]
        shutil.copy('ParameterSpace.pdf', self.test.feature_set.parameters[key_outfile])

    def perform_test_michelixtrace(self):
        from spring.tests.micprgs.test_michelixtrace import TestMicHelixTraceMain
        self.test = TestMicHelixTraceMain()
        self.test.setup()
        self.test.do_test_case_mt1()
        
    def perform_test_micctfdetermine(self):
        from spring.tests.micprgs.test_micctfdetermine import TestMicCtfDetermineMain
        self.test = TestMicCtfDetermineMain()
        self.test.setup()
        self.test.do_test_case_scd1()
        
    def perform_test_scanrowcolcorr(self):
        from spring.tests.micprgs.test_scanrowcolcorr import TestScanRowColCorrMain
        self.test = TestScanRowColCorrMain()
        self.test.setup()
        self.test.do_test_case_rcc1()

    def perform_test_scandotfit(self):
        from spring.tests.micprgs.test_scandotfit import TestScanDotFitMain
        self.test = TestScanDotFitMain()
        self.test.setup()
        self.test.do_test_case_df1()

    def perform_test_scanlinefit(self):
        from spring.tests.micprgs.test_scanlinefit import TestScanLineFitMain
        self.test = TestScanLineFitMain()
        self.test.setup()
        self.test.do_test_case_lf1()

class GalleryTestsSegment2d(object):
    def perform_test_seglayer2lattice(self):
        from spring.tests.segment2d.test_seglayer2lattice import TestSegLayer2LatticeMain
        self.test = TestSegLayer2LatticeMain()
        self.test.setup()
        self.test.do_test_case_sl2l_1()
        
        key_infile = list(self.test.feature_set.parameters.keys())[0]
        self.test.feature_set.parameters[key_infile] = 'test_power.hdf'
        self.copy_file_to_output('seglayer2lattice.png') 
        
    def perform_test_segmentalign2d(self):
        from spring.tests.segment2d.test_segmentalign2d import TestSegmentAlign2dMain
        self.test = TestSegmentAlign2dMain()
        self.test.setup()
        self.test.do_test_case_sa1()
        
    def perform_test_segment(self):
        from spring.tests.segment2d.test_segment import TestSegmentMain
        self.test = TestSegmentMain()
        self.test.setup()
        self.test.do_test_case_seg1()

    def perform_test_segmentexam(self):
        from spring.tests.segment2d.test_segmentexam import TestSegmentExamMain
        self.test = TestSegmentExamMain()
        self.test.setup()
        self.test.do_test_case_se1()

    def perform_test_segmentclass(self):
        from spring.tests.segment2d.test_segmentclass import TestSegmentClassMain
        self.test = TestSegmentClassMain()
        self.test.setup()
        self.test.do_test_case_sc1()

    def perform_test_segclassexam(self):
        from spring.tests.segment2d.test_segclassexam import TestSegClassExamMain
        self.test = TestSegClassExamMain()
        self.test.setup()
        self.test.do_test_case_sce1()

    def perform_test_segclasslayer(self):
        from spring.tests.segment2d.test_segclasslayer import TestSegClassLayerMain
        self.test = TestSegClassLayerMain()
        self.test.setup()
        self.test.do_test_case_scl1()
        self.copy_file_to_output('segclasslayer.png') 
        
    def perform_test_segmentselect(self):
        from spring.tests.segment2d.test_segmentselect import TestSegmentSelectMain
        self.test = TestSegmentSelectMain()
        self.test.setup()
        self.test.do_test_case_scex1()

    def copy_file_to_output(self, outfile):
        key_outfile = list(self.test.feature_set.parameters.keys())[1]
        self.test.feature_set.parameters[key_outfile] = outfile
        shutil.copy(os.path.join(os.pardir, os.pardir, 'screenshots', outfile), outfile)

    def perform_test_segmentplot(self):
        from spring.tests.segment2d.test_segmentplot import TestSegmentPlotMain
        self.test = TestSegmentPlotMain()
        self.test.setup()
        self.test.do_test_case_sp1()
        self.copy_file_to_output('segmentplot.png') 

class GalleryTestsSegment3d(object):
    def perform_test_segrefine3dcyclexplore(self):
        from spring.tests.segment3d.test_segrefine3dcyclexplore import TestSegRefine3dCycleExploreMain
        self.test = TestSegRefine3dCycleExploreMain()
        self.test.setup()
        
        self.test.do_test_case_sr3dce1()
        os.chdir(os.pardir)
        self.copy_file_to_output('segrefine3dcyclexplore.png') 

    def perform_test_segrefine3dinspect(self):
        from spring.tests.segment3d.test_segrefine3dinspect import TestSegRefine3dInspectMain
        self.test = TestSegRefine3dInspectMain()
        self.test.setup()
        self.test.do_test_case_sr3di1()
        self.copy_file_to_output('segrefine3dinspect.png') 

    def perform_test_segrefine3dplot(self):
        from spring.tests.segment3d.test_segrefine3dplot import TestSegRefine3dPlotMain
        self.test = TestSegRefine3dPlotMain()
        self.test.setup()
        
        self.test.do_test_case_sr3dp1()
        os.chdir(os.pardir)
        self.copy_file_to_output('segrefine3dplot.png') 

    def perform_test_segmentrefine3d(self):
        from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dMain
        self.test = TestSegmentRefine3dMain()
        self.test.setup()
        self.test.do_test_case_sr1()
        if self.program == 'segrefine3dgrid':
            from spring.tests.segment3d.test_segrefine3dgrid import TestSegRefine3dGridMain
            self.test = TestSegRefine3dGridMain()
            self.test.setup()
            
            os.chdir(os.pardir)
            files = glob('{0}/*'.format(self.test.testingdir.testdir))
            [os.rename(each_file, os.path.basename(each_file)) for each_file in files]
            os.rmdir(self.test.testingdir.testdir)

        
    def perform_test_segmentctfapply(self):
        from spring.tests.segment2d.test_segmentctfapply import TestSegmentCtfApplyMain
        self.test = TestSegmentCtfApplyMain()
        self.test.setup()
        self.test.do_test_case_sca1()
        
    def perform_test_seggridexplore(self):
        from spring.tests.segment3d.test_seggridexplore import TestSegGridExploreMain
        self.test = TestSegGridExploreMain()
        self.test.setup()
        
        self.test.do_test_sgx1()
        os.chdir(os.pardir)
        self.copy_file_to_output('seggridexplore.png') 
        
        
    def perform_test_segclassreconstruct(self):
        from spring.tests.segment3d.test_segclassreconstruct import TestSegClassReconstructMain
        self.test = TestSegClassReconstructMain()
        self.test.setup()
        self.test.do_test_case_cr1()
        
        
    def perform_test_segclassmodel(self):
        from spring.tests.segment3d.test_segclassmodel import TestSegClassModelMain
        self.test = TestSegClassModelMain()
        self.test.setup()
        self.test.do_test_case_scm1()
        
        
    def perform_test_segmultirefine3d(self):
        from spring.tests.segment3d.test_segmultirefine3d import TestSegMultiRefine3dMain
        self.test = TestSegMultiRefine3dMain()
        self.test.setup()
        self.test.do_test_case_smr3d1()
        key_infile = list(self.test.feature_set.parameters.keys())[1]
        self.test.feature_set.parameters[key_infile] = 'test_recvol.hdf'
        
        
class GalleryTests(GalleryTestSetup, GalleryTestsMicrograph, GalleryTestsSegment2d, GalleryTestsSegment3d):
    """
    * Class to perform test procedures
    """
    def perform_test_particleclass(self):
        from spring.tests.particle2d.test_particleclass import TestParticleClass
        self.test = TestParticleClass()
        self.test.setup()
        self.test.do_test_case_pc1()
        
    def perform_test_cleanup(self):
        if self.program in ['segrefine3dgrid']:
            testing_dir = os.path.join(self.current_directory, self.program, 'test_segmentrefine3d')
            to_be_deleted = os.listdir(testing_dir)
            [os.remove(each_file) for each_file in to_be_deleted]
            os.rmdir(testing_dir)
        else:
            self.test.teardown()

class GalleryBrowserFiles(object):
    """
    * Class that holds functions for rst file write
    """
    def open_gallery_rst_file(self):
        if not os.path.isdir('programs'):
            os.mkdir('programs')
        self.rst_file = open(os.path.join('programs', 'gallery' + os.extsep + 'rst'), 'w')
        self.rst_file.write('.. _gallery:\n\n')
        self.rst_file.write('Gallery\n' + 40*'=' + '\n')

        if not os.path.isdir('gallery_images'):
            os.mkdir('gallery_images')
            
    def convert_file_for_webbrowser(self, filename):
        if self.program in ['segrefine3dgrid']:
            self.filename = os.path.join(self.current_directory, self.program, 'test_segmentrefine3d',
            os.path.basename(filename))
        else:
            self.filename = os.path.join(self.current_directory, self.program, self.test.testingdir.testdir,
            os.path.basename(filename))

        self.filename_extension = self.filename.split(os.extsep)[-1]
        self.in_or_outputtype = next(self.fileit)        
        if self.filename_extension in ['hdf', 'spi', 'img', 'hed', 'mrc', 'tif']:
            self.combine_EMData_to_img()
            self.write_img_to_png()
        elif self.filename_extension in ['png', 'pdf']:
            self.copy_png_to_png()
        return self.converted_file

    def combine_EMData_to_img(self, filename=None):
        """
        >>> import numpy as np
        >>> imgarr = np.array([[ 0.,  0.,  1.,  1.],
        ...                    [ 0.,  0.,  1.,  0.],
        ...                    [ 0.,  1.,  1.,  0.],
        ...                    [ 1.,  1.,  0.,  0.]])
        >>> from EMAN2 import EMNumPy
        >>> from spring.csinfrastr.csdocu import GalleryBrowserFiles
        >>> filename = 'test_stack.hdf'
        >>> def setup_and_merge_stack_with_image_number(imgarr, number):
        ...     for image_number in range(number):
        ...         EMNumPy.numpy2em(imgarr).write_image(filename, image_number)
        ...     img = GalleryBrowserFiles().combine_EMData_to_img(filename)
        ...     os.remove(filename)
        ...     return np.copy(EMNumPy.em2numpy(img))
        >>> setup_and_merge_stack_with_image_number(imgarr, 6)
        array([[ 0.,  0.,  1.,  1.,  0.,  0.,  1.,  1.],
               [ 0.,  0.,  1.,  0.,  0.,  0.,  1.,  0.],
               [ 0.,  1.,  1.,  0.,  0.,  1.,  1.,  0.],
               [ 1.,  1.,  0.,  0.,  1.,  1.,  0.,  0.],
               [ 0.,  0.,  1.,  1.,  0.,  0.,  1.,  1.],
               [ 0.,  0.,  1.,  0.,  0.,  0.,  1.,  0.],
               [ 0.,  1.,  1.,  0.,  0.,  1.,  1.,  0.],
               [ 1.,  1.,  0.,  0.,  1.,  1.,  0.,  0.],
               [ 0.,  0.,  1.,  1.,  0.,  0.,  1.,  1.],
               [ 0.,  0.,  1.,  0.,  0.,  0.,  1.,  0.],
               [ 0.,  1.,  1.,  0.,  0.,  1.,  1.,  0.],
               [ 1.,  1.,  0.,  0.,  1.,  1.,  0.,  0.]], dtype=float32)
        >>> imgarr = np.array([0])
        >>> setup_and_merge_stack_with_image_number(imgarr, 2).shape
        (2,)
        >>> setup_and_merge_stack_with_image_number(imgarr, 6).shape
        (3, 2)
        >>> setup_and_merge_stack_with_image_number(imgarr, 12).shape
        (3, 4)
        >>> setup_and_merge_stack_with_image_number(imgarr, 25).shape
        (3, 4)
        """
        if filename is None: filename = self.filename
        self.filename = filename

        self.img = EMData()
        self.number_of_images_on_stack = EMUtil.get_image_count(self.filename)
        if  self.number_of_images_on_stack == 1:
            self.img.read_image(self.filename)
        elif 1 < self.number_of_images_on_stack <= 12:
            self.increment = 1
            self.tile_ten_images_to_montage()
        elif self.number_of_images_on_stack > 12:
            self.increment = int(self.number_of_images_on_stack/12)
            self.number_of_images_on_stack = 12
            self.tile_ten_images_to_montage()

        return self.img

    def copy_png_to_png(self):
        self.converted_file = '{0}{1}gallery_images{2}{3}_{4}{5}{6}'.format(self.current_directory, os.sep, os.sep,
        self.program, self.in_or_outputtype, os.extsep, self.filename_extension)
        
        if not os.path.isfile(self.filename):
            files = glob('{0}*{1}'.format(os.path.splitext(self.filename)[0], os.path.splitext(self.filename)[-1]))
            shutil.copy(files[0], self.converted_file) 
        else:
            shutil.copy(self.filename, self.converted_file) 

    def tile_ten_images_to_montage(self):

        if self.number_of_images_on_stack >= 6:
            column_count = int(self.number_of_images_on_stack/3)
            row_count = int(self.number_of_images_on_stack/(column_count))
        else:
            row_count = 1
            column_count = self.number_of_images_on_stack
        images = {}
        horizontal_stacks = {}
        image_number = 0
        self.img.read_image(self.filename, 0)
        padsize_x_to_include_border = int(1.1*self.img.get_xsize())
        padsize_y_to_include_border = int(1.1*self.img.get_ysize())
        for row_number in range(row_count):
            for column_number in range(column_count):
                new_image_number = image_number*self.increment
                self.img.read_image(self.filename, new_image_number)
                padimg = Util.pad(self.img, padsize_x_to_include_border, padsize_y_to_include_border, 1, 0, 0, 0, '0')
                images['column{0}'.format(column_number)] = np.copy(EMNumPy.em2numpy(padimg))
                image_number += 1
            horizontal_stacks['row{0}'.format(row_number)] = \
            np.hstack([images['column{0}'.format(column_number)] for column_number in range(column_count)])
            
        if column_count >= 1:
            vertical_stack = np.vstack([horizontal_stacks['row{0}'.format(row_number)] 
                                        for row_number in range(row_count)])
        elif column_count == 1:
            vertical_stack = horizontal_stacks['row0']

        self.img = EMNumPy.numpy2em(np.copy(vertical_stack))

    def write_img_to_png(self):
        self.converted_file = '{0}{1}gallery_images{2}{3}_{4}{5}png'.format(self.current_directory, os.sep, os.sep,
        self.program, self.in_or_outputtype, os.extsep)
        
        self.img.write_image(self.converted_file)

    def add_program_header(self):
        header_referenced = """
        \n:ref:`{program}`\n{underline}
        \n.. automodule:: {first_code}
        """.format(program = self.program, underline=(len(self.program) + 7) * '^', 
        first_code=self.test.feature_set.code_files[0])

        self.rst_file.write(header_referenced)

    def add_images_to_gallery(self):
        relative_infile = os.path.join(os.pardir, 'gallery_images', os.path.basename(self.browser_infile))
        
        relative_outfile = os.path.join(os.pardir, 'gallery_images', os.path.basename(self.browser_outfile))
        
#         relative_infile = '{pardir}{separator}gallery_images{separator}{basename_infile}'.format(pardir=os.pardir, 
#         separator=os.sep, basename_infile=os.path.basename(self.browser_infile))
#         
#         relative_outfile = '{pardir}{separator}gallery_images{separator}{basename_outfile}'.format(pardir=os.pardir, 
#         separator=os.sep, basename_outfile=os.path.basename(self.browser_outfile))
        
        basename_infile = os.path.basename(self.browser_infile)
        basename_outfile = os.path.basename(self.browser_outfile)
        basename_infile_border='|{0}|'.format(os.path.basename(self.browser_infile))
        basename_outfile_border='|{0}|'.format(os.path.basename(self.browser_outfile))

        out = list(self.parameters.keys())[1]
        if out == 'Batch mode':
            out = 'Interactive screenshot'
            
        figure_table = """
        +{horizontal_border:<49}+{horizontal_border:<49}+
        {vert_border} {basename_infile_border:<49}{vert_border} {basename_outfile_border:<49}{vert_border}
        +{horizontal_border:<49}+{horizontal_border:<49}+
        {vert_border:<2}Input: {input_parameter:<42}{vert_border:<2}Output: {output_parameter:<41}{vert_border}
        +{horizontal_border:<49}+{horizontal_border:<49}+
        
        .. {basename_infile_border} image:: {relative_infile}
            :alt: tree
            :width: 4 cm
            :target: {pardir}{separator}_images{separator}{basename_infile}
            
        .. {basename_outfile_border} image:: {relative_outfile}
            :alt: tree
            :width: 4 cm
            :target: {pardir}{separator}_images{separator}{basename_outfile}

        """.format(basename_infile = basename_infile, basename_outfile=basename_outfile,
        basename_infile_border = basename_infile_border, basename_outfile_border=basename_outfile_border,
        relative_infile=relative_infile, relative_outfile=relative_outfile,
        input_parameter=list(self.parameters.keys())[0], output_parameter=out,
        horizontal_border=50 * '-', vert_border='|', separator=os.sep, pardir=os.pardir)

        self.rst_file.write(figure_table)

class Gallery(GalleryTests, GalleryBrowserFiles):
    """
    * Class to generate gallery from input and output of each individual program
    """

    def __init__(self):
        self.open_gallery_rst_file()
        self.current_directory = os.path.abspath(os.curdir)
         
        self.prgms_dir = 'programs'
        if not os.path.isdir(self.prgms_dir):
            os.mkdir(self.prgms_dir)
        self.run_micprgs_tests()
        self.run_segment2d_tests()
        self.run_segment3d_tests()
        
#        self.write_updated_changelog()

    def run_test(self, program):
        self.program = program
        self.prepare_tests_in_new_directory()

        self.perform_test_program()
        self.copy_input_and_output_files_to_images()
        os.chdir(os.path.join(self.current_directory, 'gallery_images'))

        self.fileit = iter(['input', 'output'])
        self.browser_infile = self.convert_file_for_webbrowser(self.infile)
        self.browser_outfile = self.convert_file_for_webbrowser(self.outfile)

        self.add_program_header()
        self.add_images_to_gallery()
        if self.program in ['segrefine3dplot', 'segrefine3dcyclexplore']:
            os.chdir(os.path.join(self.current_directory, self.program, self.test.testingdir.testdir,
            'test_segmentrefine3d'))
        elif self.program in ['segrefine3dgrid']:
            os.chdir(os.path.join(self.current_directory, self.program, 'test_segmentrefine3d'))
        elif self.program in ['seggridexplore']:
            os.chdir(os.path.join(self.current_directory, self.program, self.test.testingdir.testdir,
            'test_segclassreconstruct'))
        else:
            os.chdir(os.path.join(self.current_directory, self.program, self.test.testingdir.testdir))

        self.perform_test_cleanup()
        os.chdir(self.current_directory)
        self.write_program_rst_file()
        additional_files = glob('{0}*'.format(os.path.join(self.current_directory, self.program) + os.sep))
        for each_file in additional_files:
            os.remove(each_file)
        os.rmdir(os.path.join(self.current_directory,self.program))


    def run_segment2d_tests(self):
        self.rst_file.write('\nSpring2d' + '\n' + len('Spring2d')*'-' + '\n')
        self.run_test('segment')
        self.run_test('segmentexam')
        self.run_test('segmentclass')
        self.run_test('segmentalign2d')
        self.run_test('segclassexam')
        self.run_test('segclasslayer')
        self.run_test('seglayer2lattice')
        self.run_test('segmentplot')


    def run_segment3d_tests(self):
        self.rst_file.write('\nSpring3d' + '\n' + len('Spring3d')*'-' + '\n')
        self.run_test('segmentrefine3d')
        self.run_test('segrefine3dinspect')
        self.run_test('segrefine3dgrid')
        self.run_test('seggridexplore')
        self.run_test('segmultirefine3d')
        self.run_test('segclassreconstruct')
        self.run_test('segrefine3dplot')
        self.run_test('segrefine3dcyclexplore')
        self.run_test('segclassmodel')


    def run_micprgs_tests(self):
        self.rst_file.write('\nSpringmicrograph' + '\n' + len('Springmicrograph')*'-' + '\n')
        self.run_test('scansplit')
        self.run_test('micexam')
        self.run_test('michelixtrace')
        self.run_test('michelixtracegrid')
        self.run_test('micctfdetermine')
        self.run_test('scanrowcolcorr')
        self.run_test('scandotfit')
        self.run_test('scanlinefit')

    def write_updated_changelog(self):
        changelog = open('changelog.rst', 'w')
        changelog_header = """.. _changelog:
        \nChangelog\n=========\n
        """
        changelog.write(changelog_header)
        
        proc = subprocess.Popen(['hg', 'log', '--style=changelog', sys.argv[0]], shell=False,
                                stdout=subprocess.PIPE)
        changelog.write(proc.communicate()[0])
        changelog.close()
    
    def perform_test_program(self):
        if self.program == 'segment':
            self.perform_test_segment()
        elif self.program == 'segmentexam':
            self.perform_test_segmentexam()
        elif self.program == 'segmentclass':
            self.perform_test_segmentclass()
        elif self.program == 'segclassexam':
            self.perform_test_segclassexam()
        elif self.program == 'segmentalign2d':
            self.perform_test_segmentalign2d()
        elif self.program == 'segclasslayer':
            self.perform_test_segclasslayer()
        elif self.program == 'seglayer2lattice':
            self.perform_test_seglayer2lattice()
        elif self.program == 'segmentselect':
            self.perform_test_segmentselect()
        elif self.program == 'segmentplot':
            self.perform_test_segmentplot()
            
        elif self.program == 'segmentrefine3d':
            self.perform_test_segmentrefine3d()
        elif self.program == 'segrefine3dgrid':
            self.perform_test_segmentrefine3d()
        elif self.program == 'segrefine3dinspect':
            self.perform_test_segrefine3dinspect()
        elif self.program == 'segrefine3dcyclexplore':
            self.perform_test_segrefine3dcyclexplore()
        elif self.program == 'segrefine3dplot':
            self.perform_test_segrefine3dplot()
        elif self.program == 'segmentctfapply':
            self.perform_test_segmentctfapply()
        elif self.program == 'segclassreconstruct':
            self.perform_test_segclassreconstruct()
        elif self.program == 'segclassmodel':
            self.perform_test_segclassmodel()
        elif self.program == 'segmultirefine3d':
            self.perform_test_segmultirefine3d()
        elif self.program == 'seggridexplore':
            self.perform_test_seggridexplore()
        elif self.program == 'particleclass':
            self.perform_test_particleclass()
            
        elif self.program == 'scansplit':
            self.perform_test_scansplit()
        elif self.program == 'micexam':
            self.perform_test_micexam()
        elif self.program == 'michelixtrace':
            self.perform_test_michelixtrace()
        elif self.program == 'michelixtracegrid':
            self.perform_test_michelixtracegrid()
        elif self.program == 'micctfdetermine':
            self.perform_test_micctfdetermine()
        elif self.program == 'scanrowcolcorr':
            self.perform_test_scanrowcolcorr()
        elif self.program == 'scandotfit':
            self.perform_test_scandotfit()
        elif self.program == 'scanlinefit':
            self.perform_test_scanlinefit()

        self.parameters = self.test.feature_set.parameters


    def write_program_rst_file(self):
        command_line = '{0} --print_rst {1} {2}'.format(self.program, self.browser_infile, self.browser_outfile)
        print(command_line)
        subprocess.call(command_line.split())
        
        rst_file = self.program + os.extsep + 'rst'
        os.rename(rst_file, os.path.join(self.prgms_dir, rst_file))
        prg_name_file = self.program + '_functions' + os.extsep + 'rst'
        os.rename(prg_name_file, os.path.join(self.prgms_dir, prg_name_file))
        
        
class RecordRst(GalleryBrowserFiles):
    """
    Class that holds functions required for rst file generation and sphinx build generation
    """
    def __init__(self, parset):
        """
        Initialize rstfile for further modification
        """
        self.progname = parset.progname
        self.proginfo = parset.proginfo
        self.parameters = parset.parameters
        self.level = parset.level
        self.program_states = parset.program_states
        self.hints = parset.hints
        self.properties = parset.properties
        self.parser = parset.parser
        self.code_files = parset.code_files
        options  = self.parser.parse_args()
        self.args = options.input_output

        self.rst_file = open(self.progname + '.rst', 'w')

        self.mkintrorst()
        if len(self.args) == 2:
            self.add_input_output_table()
        
        levels = Support().define_levels_beginner_intermediate_expert()
        for self.this_level in ['beginner', 'intermediate', 'expert']:
            self.this_levels = levels[self.this_level]
            self.inputrst()
            self.write_sample_parameter_file()
            
        self.cmdlineoptrst()
        self.programstr()
        self.generate_functions_file()

        sys.exit()

    def mkintrorst(self):
        """
        Brief program description at the top of the page
        """
        self.rst_file.write('.. _'+ self.progname + ':\n\n')
        self.rst_file.write(self.progname.capitalize() + '\n' + len(self.progname)*'-' + '\n')
#        self.rst_file.write('\n.. automodule:: %s\n' %(self.progname))
        self.rst_file.write('\n{0}\n'.format(self.proginfo))

    def add_input_output_table(self):
        self.browser_infile = self.args[0]
        self.browser_outfile = self.args[1]
        self.program = self.progname
        self.add_images_to_gallery()


    def print_help_message_with_indentation(self):
        this_file = open('thisfile.txt', 'w')
        self.parser.print_help(this_file)
        this_file.close()
        written_file = open('thisfile.txt', 'r')
        indented_string = ''
        for each_line in written_file.readlines():
            indented_string += '    {0}'.format(each_line)
        
        written_file.close()
        os.remove('thisfile.txt')
        self.rst_file.write(indented_string)
        self.rst_file.close()

    def cmdlineoptrst(self):
        self.rst_file.write('\n.. _using_%s_on_cmdline:\n' %self.progname)
        self.rst_file.write('\nCommand line options' + '\n' + 23*'~' + '\n')
        self.rst_file.write('\nWhen invoking ' + self.progname + ', you may specify any of these options::\n\n')

        self.print_help_message_with_indentation()

        # add tabs to 'Usage' lines for correct cmdline formatting
        rfile = open(self.progname + '.rst', 'r')  # r for reading
        lines = rfile.readlines()
        rfile.close()
        
        self.rst_file = open(self.progname + '.rst', 'w')
        for line in lines:
            if line.strip().startswith('Usage:'):
                self.rst_file.write('    ' + line)
            elif line.strip().startswith('Options:'):
                self.rst_file.write(line.strip() + ':\n\n')
            else:
                self.rst_file.write(line)
                

    def inputrst(self):
        self.rst_file.write('\n\n.. _making_a_{0}_table_for_{1}:\n\n'.format(self.progname, self.this_level))
        heading = 'parameters'
        if self.this_level not in ['beginner']:
            heading = 'additional ' + heading + ' ({0} level)'.format(self.this_level) 
        self.rst_file.write(heading.capitalize() + '\n' + 45 * '~' + '\n\n')
        self.rst_file.write('.. tabularcolumns:: |p{4cm}|p{2.5cm}|p{9cm}|\n\n')
        self.rst_file.write('%-80s %-80s %s\n' %(78*'=', 78*'=', 100*'='))
        self.rst_file.write('%-80s %-80s %s\n' %('Parameter', 'Example (default)', 'Description'))
        self.rst_file.write('%-80s %-80s %s\n' %(78*'=', 78*'=', 100*'='))
        for name in self.hints:
            if self.level[name] == self.this_level:
                if type(self.parameters[name]) in [int, float, tuple]:
                    if self.hints[name].endswith('.'):
                        hint_str = self.hints[name][:-1]
                    else:
                        hint_str = self.hints[name]
                    helpstr = hint_str + \
                    ' (accepted values min=%g, max=%g).' %(self.properties[name].minimum, self.properties[name].maximum)
                else:
                    helpstr = self.hints[name]
                    if helpstr.find('\n'):
                        helpstr = '; '.join(helpstr.split('\n'))
                self.rst_file.write('%-80s %-80s %s\n' % (name, self.parameters[name], helpstr))
        self.rst_file.write('%-80s %-80s %s\n\n' %(78*'=', 78*'=', 100*'='))


    def write_sample_parameter_file(self):
        self.rst_file.write('\n\n.. _{0}_parameter_file_for_{1}:'.format(self.progname, self.this_level))
        heading = '\n\nSample parameter file'
        if self.this_level not in ['beginner']:
            heading = heading + ' ({0} level)'.format(self.this_level)
        self.rst_file.write(heading + '\n')
        self.rst_file.write('%-60s\n' %(58*'~'))
        self.rst_file.write('\nYou may run the program in the command line by providing the parameters via a text file::')
        self.rst_file.write('\n\n    %s --f parameterfile.txt\n' %self.progname)
        self.rst_file.write('\nWhere the format of the parameters is::\n\n')
        for name in self.parameters:
            if self.level[name] in self.this_levels:
                self.rst_file.write('    %-40s = %s\n' % (name, self.parameters[name]))


    def programstr(self):
        self.rst_file.write(2*'\n' + 'Program flow\n' + 20*'~' + '\n')
        for state in self.program_states:
            self.rst_file.write('#. {0}: {1}\n'.format(state, self.program_states[state]))
        self.rst_file.write(2*'\n' + 'Functions\n' + 20*'~' + '\n')
        self.rst_file.write('\n.. toctree::\n')
        self.rst_file.write('   :maxdepth: 1\n')
        self.codefile_name = '{0}_functions{1}rst'.format(self.progname, os.extsep)
        self.rst_file.write('\n   {0}'.format(os.path.splitext(self.codefile_name)[0]))
        self.rst_file.close()

    def generate_functions_file(self):
        self.code_file = open(self.codefile_name, 'w')
        self.code_file.write('.. _{0}:\n'.format(os.path.splitext(self.codefile_name)[0]))
        self.code_file.write('\nDocumented functions with links to source\n' + 45 * '~' + '\n')
        for each_file in self.code_files:
            self.code_file.write('\n.. automodule:: {0}\n'.format(each_file))
            self.code_file.write('    :members:\n')
            self.code_file.write('    :undoc-members:\n\n')
        
        self.code_file.close()

def main():
    gal = Gallery()
        
if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check segclassreconstruct
"""
from EMAN2 import EMData, Transform, Util
from fundamentals import fshift, rot_shift2D
from glob import glob
from multiprocessing import cpu_count
import os
from projection import prj
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.scansplit import Micrograph
from spring.segment3d.segclassreconstruct import SegClassReconstructPar, SegClassReconstruct
from spring.springgui.spring_launch import DataBaseUpdate
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
import sys
from utilities import model_circle, model_gauss_noise, model_square

from nose.tools import raises


class TestSegClassReconstructPreparation(object):

    def shift_volume_and_helicise(self, pixelsize, helix_radius_in_angstrom, helical_rise, helical_rotation, vol):
        helix_radius = helix_radius_in_angstrom / pixelsize
        subunit_volume = fshift(vol, helix_radius)
        helix_volume = subunit_volume.helicise(pixelsize, helical_rise, -helical_rotation, 0.8)
        return helix_volume


    def generate_helical_volume_square(self, size_of_subunit_in_angstrom, pixelsize, larger_dimension,
    helix_radius_in_angstrom, helical_rise, helical_rotation):
        radius = size_of_subunit_in_angstrom/(2*pixelsize)
        s = model_square(radius, larger_dimension, larger_dimension, larger_dimension)
        t1 = Transform({'type':'spider', 'psi':33, 'theta':100, 'phi':77, 'scale':1.0})
        small_radius = radius/2.0
        ss = model_square(small_radius, larger_dimension, larger_dimension, larger_dimension)
        ss.transform(t1)
        vol = s - ss
        
        helix_volume = self.shift_volume_and_helicise(pixelsize, helix_radius_in_angstrom, helical_rise,
        helical_rotation, vol)
        
#        helix_volume.write_image('test_helix.hdf')
        
        return helix_volume


    def generate_helical_volume(self, size_of_subunit_in_angstrom, pixelsize, larger_dimension,
    helix_radius_in_angstrom, helical_rise, helical_rotation):
        radius = size_of_subunit_in_angstrom/(2*pixelsize)
        vol = model_circle(radius, larger_dimension, larger_dimension, larger_dimension)
        
        helix_volume = self.shift_volume_and_helicise(pixelsize, helix_radius_in_angstrom, helical_rise,
        helical_rotation, vol)
        
#        helix_volume.write_image('test_helix.hdf')
        
        return helix_volume


    def generate_noise_projections_from_helix(self, projection_file, helix_volume, projection_parameters):
        
        helical_projections = prj(helix_volume, projection_parameters)
        
        larger_dimension = helix_volume.get_xsize()
        noise_projections = []
        for each_segment_index, each_helical_projection in enumerate(helical_projections):
            stat = Micrograph().get_statistics_from_image(each_helical_projection)
            noise = model_gauss_noise(0.5 * stat.sigma, larger_dimension, larger_dimension)
            each_helical_projection += noise
            noise_projections.append(each_helical_projection)
            if projection_file is not None:
                each_helical_projection.write_image(projection_file, each_segment_index)

        return noise_projections
    
    
    def prepare_helical_projection(self, projection_file, rotational_sym=1):
        length_of_helix_in_pixels = int(self.volume_size_in_angstrom/self.pixelsize)
        self.percent_reconstruction_size = 50
        larger_dimension = int(round(100*length_of_helix_in_pixels/self.percent_reconstruction_size, -1))
        phi = 0; theta = 90; psi = 270; shift_x = 0; shift_y = 0
        projection_parameters = [[phi, theta, psi, shift_x, shift_y]]
        
        helix_volume = self.generate_helical_volume(self.size_of_subunit_in_angstrom, self.pixelsize, larger_dimension,
        self.helix_radius_in_angstrom, self.helical_rise, self.helical_rotation)
        
        if rotational_sym > 1:
            helix_volume = helix_volume.symvol('c{0}'.format(rotational_sym))
        
        projections = self.generate_noise_projections_from_helix(projection_file, helix_volume, projection_parameters)
        self.helical_projection = projections[0]
        
        return helix_volume, self.helical_projection
        
    def setup_of_common_non_mpi_parameters(self):
        self.pixelsize = 10.0
        self.projection_file = 'test_projection.hdf'
        
        self.grid_volume_name = 'grid_volume_name.hdf'
        self.test_mont_reprojection_stack = 'test_mont_repr.hdf'
        self.test_mont_power_stack = 'test_mont_power.hdf'
        self.test_bin_factor = 2
        self.helical_rise = 10.0
        self.helical_rise_increment = 4.0
        self.helical_rotation = 50.0
        self.helical_rotation_increment = 15.0
        self.size_of_subunit_in_angstrom = 20
        self.helix_radius_in_angstrom = 50
        self.volume_size_in_angstrom = 500
        self.helix_offcenter_shift = 4
        self.helix_rotation = 6
        
        self.prepare_helical_projection(self.projection_file)
        self.feature_set.parameters['Class average stack'] = self.projection_file
        self.feature_set.parameters['Volume name']=self.grid_volume_name
#        self.feature_set.parameters['Image output stack'] = self.reprojection_file
        self.feature_set.parameters['Keep intermediate files']=True
        self.feature_set.parameters['Montage stack of class average vs. reprojection ' + \
        'images']=self.test_mont_reprojection_stack
        
        self.feature_set.parameters['Montage stack of class average vs. reprojection power ' + \
        'spectra']=self.test_mont_power_stack
        
        self.feature_set.parameters['Class number to be analyzed'] = 0
        self.feature_set.parameters['Pixel size in Angstrom'] = self.pixelsize
        
        self.feature_set.parameters['Binning option'] = True
        self.feature_set.parameters['Binning factor'] = self.test_bin_factor

        self.feature_set.parameters['Estimated helix inner and outer diameter in ' + \
        'Angstrom']=((int(0.5*self.helix_radius_in_angstrom), int(3.3*self.helix_radius_in_angstrom)))
        
        self.feature_set.parameters['Helical rise/rotation or pitch/number of units per turn choice']='rise/rotation'
        
        self.feature_set.parameters['Range of helical rise or pitch search in Angstrom'] = (self.helical_rise - \
        self.helical_rise_increment, self.helical_rise + self.helical_rise_increment)
        
        self.feature_set.parameters['Range of helical rotation in degrees or number of units per turn search'] = \
        (self.helical_rotation - self.helical_rotation_increment, self.helical_rotation +
        self.helical_rotation_increment)
        
        self.feature_set.parameters['Increments of helical symmetry steps in Angstrom or degrees'] = \
        (self.helical_rise_increment, self.helical_rotation_increment/3)
        
        self.feature_set.parameters['Rotational symmetry']=1
        self.feature_set.parameters['Helix polarity']='polar'
        self.feature_set.parameters['Out-of-plane tilt angle']=0
        
        self.feature_set.parameters['Percent of image reconstruction size'] = self.percent_reconstruction_size
        self.feature_set.parameters['Minimum number of images for 3D reconstruction']=int(8)
        self.feature_set.parameters['Center option']=True
        self.feature_set.parameters['Local symmetry grid search']=False
        
        
class TestSegClassReconstruct(TestSegClassReconstructPreparation, SegClassReconstruct):

    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = SegClassReconstructPar()
        self.setup_of_common_non_mpi_parameters()
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)
            
        super(TestSegClassReconstruct, self).__init__(self.feature_set)
        
    def produce_a_projection_of_final_volume(self):
        symmetry_helix_volume = self.get_symmetry_file_name((self.helical_rise, self.helical_rotation), self.volume_name)
        last_volume = EMData()
        last_volume.read_image(symmetry_helix_volume)
        
        projection_parameters = [[0., 90., 270., 0., 0.]]
        helical_projections = self.project_locally(last_volume, projection_parameters, self.volume_name)
        

    def teardown(self):
        files_to_be_deleted = [self.projection_file, self.test_mont_power_stack, self.test_mont_reprojection_stack,
        'grid.db']

        files_to_be_deleted +=glob('{0}*'.format(os.path.splitext(self.grid_volume_name)[0]))
        for each_file in files_to_be_deleted:
            os.remove(each_file)
        
        if self.feature_set.parameters['Center option'] is True:
            centered_class_avg = '{0}_centered{1}'.format(os.path.splitext(self.projection_file)[0],
            os.path.splitext(self.projection_file)[-1])
            
            os.remove(centered_class_avg)

        self.testingdir.remove()


class TestSegClassReconstructMain(TestSegClassReconstruct):
    def do_test_case_cr1(self):
        self.perform_reconstructions_from_class_for_symmetry_combinations()
         
        self.produce_a_projection_of_final_volume()
            

class TestSegClassReconstructMore(TestSegClassReconstruct):
    def check_and_remove_symgrid_volumes(self, grid_volume_name):
        for each_symmetry_pair in self.symmetry_sequence:
            symgrid_volume = self.generate_symmetry_grid_file_name(each_symmetry_pair, grid_volume_name,
            self.rise_rot_or_pitch_unit_choice)
            
            assert os.path.isfile(symgrid_volume)
    
    @raises(ValueError)
    def do_impossible_crtest(self):
        self.percent_reconstruction_size = 99
        self.perform_reconstructions_from_class_for_symmetry_combinations()
         
    def do_test_case_cr0(self):
        self.perform_reconstructions_from_class_for_symmetry_combinations()
          
        self.produce_a_projection_of_final_volume()
        DataBaseUpdate().update_grid_db('grid.db', 'grid_up.db')
        os.remove('grid_up.db')
             
    def do_test_case_cr2(self):
        self.helical_projection = rot_shift2D(self.helical_projection, self.helix_rotation, self.helix_offcenter_shift, 0)
        self.helical_projection.write_image(self.projection_file)
         
        self.perform_reconstructions_from_class_for_symmetry_combinations()
        sys.stderr.write('\n{0} == {1}'.format(self.helix_offcenter_shift, -self.centerx))
        sys.stderr.write('\n{0} == {1}'.format(self.helix_rotation, -self.inplane_rotation))
#        assert self.helix_offcenter_shift == -(int(round(self.centerx)) - 1)
#        assert self.helix_rotation == int(-self.inplane_rotation + 0.5)
         
        self.check_and_remove_symgrid_volumes(self.grid_volume_name)
     
         
    def do_test_case_cr3(self):
        self.feature_set.parameters['Local symmetry grid search']=True
        super(TestSegClassReconstruct, self).__init__(self.feature_set)
        self.perform_reconstructions_from_class_for_symmetry_combinations()
         
         
    def do_test_case_cr4(self):
        self.feature_set.parameters['Binning option'] = False
        super(TestSegClassReconstruct, self).__init__(self.feature_set)
        self.perform_reconstructions_from_class_for_symmetry_combinations()


class TestSegClassReconstructEndtoEnd(TestSegClassReconstruct):
    def do_end_to_end_test_cr_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 
    def do_end_to_end_test_cr_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
class TestSegClassReconstructMpi(TestSegClassReconstruct):
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

def main():
    trc = TestSegClassReconstructMain()
    trc.setup()
    trc.do_test_case_cr1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 17-Jan-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Spring3d - suite of programs that reconstruct 3D images from segmented helical particles
"""
from PyQt4.QtGui import QMainWindow, QWidget, QTabWidget, QPixmap, QLabel, QGridLayout, QApplication, QScrollArea
from spring.csinfrastr.csgui import GUI
from spring.segment3d.refine.sr3d_main import SegmentRefine3dPar
from spring.segment3d.segclassmodel import SegClassModelPar
from spring.segment3d.segclassreconstruct import SegClassReconstructPar
from spring.segment3d.seggridexplore import SegGridExplorePar
from spring.segment3d.segmultirefine3d import SegMultiRefine3dPar
from spring.segment3d.segrefine3dgrid import SegRefine3dGridPar
from spring.segment3d.segrefine3dinspect import SegRefine3dInspectPar
from spring.segment3d.segrefine3dplot import SegRefine3dPlotPar
from spring.springgui.spring_menu import GuiActions
from textwrap import wrap
import os
import sys

class Spring3dGuiScrollTabs(QScrollArea):
    def __init__(self, features=None, parent=None):
        QWidget.__init__(self)
        
        self.features = features
        self.springTab = GUI(features)
        self.setWidget(self.springTab)
        
        
class SpringGuiTop(QWidget):
    def __init__(self, logofile=None, parent=None):
        QWidget.__init__(self)

        self.suitename = os.path.basename(sys.argv[0])
        self.setWindowTitle('%s' %self.suitename)

        self.pic = QLabel()
        self.pic.setGeometry(10, 10, 544, 100)
        #use full ABSOLUTE path to the image, not relative
        
        self.pic.setPixmap(QPixmap(os.path.join(os.path.dirname(__file__),
        '{pdir}{sep}images{sep}{logo}'.format(pdir=os.pardir, sep=os.sep, logo=logofile))))
        

class Spring3dGui(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self)

        segment3dguitop=SpringGuiTop('spring3d_logo{0}png'.format(os.extsep))

        self.tabWidget = QTabWidget()

        self.segrefine3dgrid_features = SegRefine3dGridPar()
        self.SegRefine3dGridTab = Spring3dGuiScrollTabs(self.segrefine3dgrid_features)

        self.segmentrefine3d_features = SegmentRefine3dPar()
        self.segmentRefine3dTab = Spring3dGuiScrollTabs(self.segmentrefine3d_features)

        self.seggridexplore_features = SegGridExplorePar()
        self.segGridExploreTab = Spring3dGuiScrollTabs(self.seggridexplore_features)
        
        self.segrefine3dplot_features= SegRefine3dPlotPar()
        self.segRefine3dPlotTab = Spring3dGuiScrollTabs(self.segrefine3dplot_features)
        
        self.segclassreconstruct_features = SegClassReconstructPar()
        self.segClassReconstructTab = Spring3dGuiScrollTabs(self.segclassreconstruct_features)
        
        self.segrefine3dinspect_features = SegRefine3dInspectPar()
        self.segRefine3dInspectTab = Spring3dGuiScrollTabs(self.segrefine3dinspect_features)

        self.segclassmodel_features = SegClassModelPar()
        self.segClassModelTab = Spring3dGuiScrollTabs(self.segclassmodel_features)

        self.segmultirefine3d_features = SegMultiRefine3dPar()
        self.segMultiRefine3dTab = Spring3dGuiScrollTabs(self.segmultirefine3d_features)

        self.addSpring3dTabs()

        layout = QGridLayout()
        layout.addWidget(segment3dguitop.pic, 0, 0, 4, 1)
        layout.addWidget(self.tabWidget, 0, 1, 7, 7)
        layout.setColumnStretch(0, 0)
        self.setLayout(layout)


    def addSpring3dTabs(self):
        self.tabWidget.addTab(self.segClassReconstructTab, self.segclassreconstruct_features.progname.title())
        self.tabWidget.setTabToolTip(0, '\n'.join(wrap(self.segclassreconstruct_features.progname.title() + ' - ' + \
        self.segclassreconstruct_features.proginfo)))
        
        self.tabWidget.addTab(self.segGridExploreTab, self.seggridexplore_features.progname.title())
        self.tabWidget.setTabToolTip(1, '\n'.join(wrap(self.seggridexplore_features.progname.title() + ' - ' + \
        self.seggridexplore_features.proginfo)))
        
        self.tabWidget.addTab(self.segmentRefine3dTab, self.segmentrefine3d_features.progname.title())
        self.tabWidget.setTabToolTip(2, '\n'.join(wrap(self.segmentrefine3d_features.progname.title() + ' - ' + \
        self.segmentrefine3d_features.proginfo)))
        
        self.tabWidget.addTab(self.SegRefine3dGridTab, self.segrefine3dgrid_features.progname.title())
        self.tabWidget.setTabToolTip(3, '\n'.join(wrap(self.segrefine3dgrid_features.progname.title() + ' - ' + \
        self.segrefine3dgrid_features.proginfo)))
        
        self.tabWidget.addTab(self.segRefine3dPlotTab, self.segrefine3dplot_features.progname.title())
        self.tabWidget.setTabToolTip(4, '\n'.join(wrap(self.segrefine3dplot_features.progname.title() + ' - ' + \
        self.segrefine3dplot_features.proginfo)))
        
        self.tabWidget.addTab(self.segRefine3dInspectTab, self.segrefine3dinspect_features.progname.title())
        self.tabWidget.setTabToolTip(5, '\n'.join(wrap(self.segrefine3dinspect_features.progname.title() + ' - ' + \
        self.segrefine3dinspect_features.proginfo)))

        self.tabWidget.addTab(self.segClassModelTab, self.segclassmodel_features.progname.title())
        self.tabWidget.setTabToolTip(6, '\n'.join(wrap(self.segclassmodel_features.progname.title() + ' - ' + \
        self.segclassmodel_features.proginfo)))

        self.tabWidget.addTab(self.segMultiRefine3dTab, self.segmultirefine3d_features.progname.title())
        self.tabWidget.setTabToolTip(7, '\n'.join(wrap(self.segmultirefine3d_features.progname.title() + ' - ' + \
        self.segmultirefine3d_features.proginfo)))


class Spring3dMain(QMainWindow, GuiActions):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)

        self.package = 'emspring'

        Spring3dCentral = Spring3dGui()
        self.setCentralWidget(Spring3dCentral)

        # inherit body of layout of actions
        self.setupUi(self)

def main():
    app = QApplication(sys.argv)
    window = Spring3dMain()

    window.show()
    app.exec_()

if __name__ == '__main__':
    main()



# Author: Carsten Sachse 8-Jul-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.segment2d.segment import Segment, SegmentPar
import os


class SegmentMpiPreparation(Segment):
    def prepare_segmentation_mpi(self):
        self.comm, self.rank, self.size, self.log, tempdir = OpenMpi().setup_mpi_and_simultaneous_logging(self.log,
        self.feature_set.logfile, self.temppath)
        
        if self.rank == 0:
            assigned_mics = self.validate_input()
            pair = self.assign_reorganize(self.micrograph_files, self.coordinate_files)

            helices, assigned_stack_ids, assigned_helix_ids = self.single_out(pair, self.stepsize, self.pixelsize,
            assigned_mics)

            helices = OpenMpi().convert_list_of_namedtuples_to_list_of_lists(helices)
            
            helices = OpenMpi().split_sequence_evenly(helices, self.size)
        
        else:
            helices = None
            assigned_stack_ids = None
            assigned_helix_ids = None
            
        helices = self.comm.scatter(helices, root=0)
        
        helixinfo = self.make_helixinfo_named_tuple()
        helices = OpenMpi().convert_list_of_lists_to_list_of_provided_namedtuple(helices, helixinfo)
        
        return helices, tempdir, assigned_stack_ids, assigned_helix_ids
            

class SegmentMpi(SegmentMpiPreparation):
    def gather_distributed_helices_to_root(self, comm, helices):
        helices = OpenMpi().convert_list_of_namedtuples_to_list_of_lists(helices)
        
        helices = comm.gather(helices, root=0)
        
        if comm.rank == 0:
            helices = OpenMpi().merge_sequence_of_sequences(helices)
            
            helixinfo = self.make_helixinfo_named_tuple()
            helices = OpenMpi().convert_list_of_lists_to_list_of_provided_namedtuple(helices, helixinfo)
        else:
            helices = None
        
        return helices
    
    
    def extract_segments_mpi(self, tempdir):
        if self.rank == 0:
            self.log.plog(10)
        imgstack = self.outfile
        local_windowed_stack = '{dir}{sep}temp_segments{rank}.hdf'.format(dir=tempdir, sep=os.sep, rank=self.rank)
        self.helices = self.cut_segments(self.helices, local_windowed_stack)
        
        if self.rank == 0:
            self.log.plog(80)
        return local_windowed_stack, imgstack


    def perform_binning_if_demanded(self, imgstack, local_windowed_stack):
        if self.binoption is True:
            local_binned_stack = '{0}binned{1}'.format(os.path.splitext(local_windowed_stack)[0],
            os.path.splitext(local_windowed_stack)[-1])
            
            binned_imgstack = '{0}-{1}xbin{2}'.format(os.path.splitext(imgstack)[0], self.binfactor,
            os.path.splitext(imgstack)[-1])
            
            if self.helices != []:
                self.binstack(local_windowed_stack, local_binned_stack, self.binfactor)
            self.comm.barrier()
            OpenMpi().gather_stacks_from_cpus_to_common_stack(self.comm, local_binned_stack, binned_imgstack)
        self.comm.barrier()


    def finish_segmentation_mpi(self, tempdir, imgstack, local_windowed_stack, assigned_stack_ids, assigned_helix_ids):
        OpenMpi().gather_stacks_from_cpus_to_common_stack(self.comm, local_windowed_stack, imgstack)
        self.helices = self.gather_distributed_helices_to_root(self.comm, self.helices)
        self.comm.barrier()
        if self.rank == 0:
            self.enter_helix_parameters_in_database(self.helices, assigned_stack_ids, assigned_helix_ids)
        os.rmdir(tempdir)
        self.comm.barrier()
        if self.rank == 0:
            self.log.endlog(self.feature_set)


    def segment(self):
        self.helices, self.tempdir, assigned_stack_ids, assigned_helix_ids = self.prepare_segmentation_mpi()
        local_windowed_stack, imgstack = self.extract_segments_mpi(self.tempdir)
        self.perform_binning_if_demanded(imgstack, local_windowed_stack)

        self.finish_segmentation_mpi(self.tempdir, imgstack, local_windowed_stack,
        assigned_stack_ids, assigned_helix_ids)
        
            
def main():
    parset = SegmentPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    stack = SegmentMpi(reduced_parset)
    stack.segment()


if __name__ == '__main__':
    main()



# Author: Carsten Sachse 17-Jan-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Springmicrograph - suite of programs that work with micrographs
"""
from PyQt4.QtGui import QMainWindow, QWidget, QTabWidget, QGridLayout, QApplication
from spring.micprgs.micctfdetermine import MicCtfDeterminePar
from spring.micprgs.micexam import MicrographExamPar
from spring.micprgs.michelixtrace import MicHelixTracePar
from spring.micprgs.scandotfit import ScanDotFitPar
from spring.micprgs.scanlinefit import ScanLineFitPar
from spring.micprgs.scanrowcolcorr import ScanRowColCorrPar
from spring.micprgs.scansplit import ScanSplitPar
from spring.springgui.spring3d_launch import SpringGuiTop, Spring3dGuiScrollTabs
from spring.springgui.spring_menu import GuiActions
from textwrap import wrap
import os
import sys


class SpringMicrographGui(QWidget):

    def addScanSpringTabs(self):
        self.tabWidget.addTab(self.scanSplitW, self.scansplit_features.progname.title())
        self.tabWidget.setTabToolTip(0, '\n'.join(wrap(self.scansplit_features.progname.title() + ' - ' + \
        self.scansplit_features.proginfo)))
        
        self.tabWidget.addTab(self.micExamW, self.micexam_features.progname.title())
        self.tabWidget.setTabToolTip(1, '\n'.join(wrap(self.micexam_features.progname.title() + ' - ' + \
        self.micexam_features.proginfo)))
        
        self.tabWidget.addTab(self.micCtfDetermineTab, self.micctfdetermine_features.progname.title())
        self.tabWidget.setTabToolTip(2, '\n'.join(wrap(self.micctfdetermine_features.progname.title() + ' - ' + \
        self.micctfdetermine_features.proginfo)))
        
        self.tabWidget.addTab(self.micHelixTraceTab, self.michelixtrace_features.progname.title())
        self.tabWidget.setTabToolTip(3, '\n'.join(wrap(self.michelixtrace_features.progname.title() + ' - ' + \
        self.michelixtrace_features.proginfo)))
        
        self.tabWidget.addTab(self.lineFitW, self.scanlinefit_features.progname.title())
        self.tabWidget.setTabToolTip(4, '\n'.join(wrap(self.scanlinefit_features.progname.title() + ' - ' + \
        self.scanlinefit_features.proginfo)))
        
        self.tabWidget.addTab(self.dotFitW, self.scandotfit_features.progname.title())
        self.tabWidget.setTabToolTip(5, '\n'.join(wrap(self.scandotfit_features.progname.title() + ' - ' + \
        self.scandotfit_features.proginfo)))
        
        self.tabWidget.addTab(self.rowColCorrW, self.scanrowcolcorr_features.progname.title())
        self.tabWidget.setTabToolTip(6, '\n'.join(wrap(self.scanrowcolcorr_features.progname.title() + ' - ' + \
        self.scanrowcolcorr_features.proginfo)))
        

    def __init__(self, parent=None):
        QWidget.__init__(self)

        springmicrographtop = SpringGuiTop('springscan_logo{0}png'.format(os.extsep))
        self.tabWidget = QTabWidget()

        self.scansplit_features = ScanSplitPar()
        self.scanSplitW = Spring3dGuiScrollTabs(self.scansplit_features)

        self.micexam_features = MicrographExamPar()
        self.micExamW = Spring3dGuiScrollTabs(self.micexam_features)

        self.micctfdetermine_features = MicCtfDeterminePar()
        self.micCtfDetermineTab = Spring3dGuiScrollTabs(self.micctfdetermine_features)

        self.michelixtrace_features = MicHelixTracePar()
        self.micHelixTraceTab = Spring3dGuiScrollTabs(self.michelixtrace_features)

        self.scanlinefit_features = ScanLineFitPar()
        self.lineFitW = Spring3dGuiScrollTabs(self.scanlinefit_features)

        self.scandotfit_features = ScanDotFitPar()
        self.dotFitW = Spring3dGuiScrollTabs(self.scandotfit_features)

        self.scanrowcolcorr_features = ScanRowColCorrPar()
        self.rowColCorrW = Spring3dGuiScrollTabs(self.scanrowcolcorr_features)

        self.addScanSpringTabs()

        layout = QGridLayout()
        layout.addWidget(springmicrographtop.pic, 0, 0, 4, 1)
        layout.addWidget(self.tabWidget, 0, 1, 7, 7)
        layout.setColumnStretch(0, 0)
        self.setLayout(layout)


class SpringMicrographMain(QMainWindow, GuiActions):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)

        self.package = 'emspring'        
        SpringMicrographCentral = SpringMicrographGui()
        self.setCentralWidget(SpringMicrographCentral)

        # inherit body of layout of actions
        self.setupUi(self)

def main():
    app = QApplication(sys.argv)
    window = SpringMicrographMain()

    window.show()
    app.exec_()

if __name__ == '__main__':
    main()

